import logging
import socket
import types
from _typeshed import ReadableBuffer
from collections.abc import Callable, Iterable, Mapping
from typing import Final, TypeVar, overload
from typing_extensions import ParamSpec, TypeAlias

__version__: Final[str]

log: logging.Logger  # undocumented

_ProxyType: TypeAlias = int

PROXY_TYPE_SOCKS4: Final[_ProxyType]
SOCKS4: Final[_ProxyType]
PROXY_TYPE_SOCKS5: Final[_ProxyType]
SOCKS5: Final[_ProxyType]
PROXY_TYPE_HTTP: Final[_ProxyType]
HTTP: Final[_ProxyType]

PROXY_TYPES: Final[dict[str, _ProxyType]]
PRINTABLE_PROXY_TYPES: Final[dict[_ProxyType, str]]

_T = TypeVar("_T")
_P = ParamSpec("_P")

def set_self_blocking(function: Callable[_P, _T]) -> Callable[_P, _T]: ...  # undocumented

class ProxyError(IOError):
    msg: str
    socket_err: socket.error
    def __init__(self, msg: str, socket_err: socket.error | None = None) -> None: ...

class GeneralProxyError(ProxyError): ...
class ProxyConnectionError(ProxyError): ...
class SOCKS5AuthError(ProxyError): ...
class SOCKS5Error(ProxyError): ...
class SOCKS4Error(ProxyError): ...
class HTTPError(ProxyError): ...

SOCKS4_ERRORS: Final[Mapping[int, str]]
SOCKS5_ERRORS: Final[Mapping[int, str]]
DEFAULT_PORTS: Final[Mapping[_ProxyType, int]]

_DefaultProxy: TypeAlias = tuple[_ProxyType | None, str | None, int | None, bool, bytes | None, bytes | None]

def set_default_proxy(
    proxy_type: _ProxyType | None = None,
    addr: str | None = None,
    port: int | None = None,
    rdns: bool = True,
    username: str | None = None,
    password: str | None = None,
) -> None: ...
def setdefaultproxy(
    proxy_type: _ProxyType | None = None,
    addr: str | None = None,
    port: int | None = None,
    rdns: bool = True,
    username: str | None = None,
    password: str | None = None,
    *,
    proxytype: _ProxyType = ...,
) -> None: ...
def get_default_proxy() -> _DefaultProxy | None: ...

getdefaultproxy = get_default_proxy

def wrap_module(module: types.ModuleType) -> None: ...

wrapmodule = wrap_module

_Endpoint: TypeAlias = tuple[str, int]

def create_connection(
    dest_pair: _Endpoint,
    timeout: int | None = None,
    source_address: _Endpoint | None = None,
    proxy_type: _ProxyType | None = None,
    proxy_addr: str | None = None,
    proxy_port: int | None = None,
    proxy_rdns: bool = True,
    proxy_username: str | None = None,
    proxy_password: str | None = None,
    socket_options: (
        Iterable[tuple[int, int, int | ReadableBuffer] | tuple[int, int, None, int]] | None
    ) = None,  # values passing to `socket.setsockopt` method
) -> socksocket: ...

class _BaseSocket(socket.socket):  # undocumented
    ...

class socksocket(_BaseSocket):
    default_proxy: _DefaultProxy | None  # undocumented
    proxy: _DefaultProxy  # undocumented
    proxy_sockname: _Endpoint | None  # undocumented
    proxy_peername: _Endpoint | None  # undocumented
    def __init__(
        self, family: socket.AddressFamily = ..., type: socket.SocketKind = ..., proto: int = 0, fileno: int | None = None
    ) -> None: ...
    def settimeout(self, timeout: float | None) -> None: ...
    def gettimeout(self) -> float | None: ...
    def setblocking(self, v: bool) -> None: ...
    def set_proxy(
        self,
        proxy_type: _ProxyType | None = None,
        addr: str | None = None,
        port: int | None = None,
        rdns: bool = True,
        username: str | None = None,
        password: str | None = None,
    ) -> None: ...
    def setproxy(
        self,
        proxy_type: _ProxyType | None = None,
        addr: str | None = None,
        port: int | None = None,
        rdns: bool = True,
        username: str | None = None,
        password: str | None = None,
        *,
        proxytype: _ProxyType = ...,
    ) -> None: ...
    def bind(self, address: socket._Address, /) -> None: ...
    @overload
    def sendto(self, bytes: ReadableBuffer, address: socket._Address) -> int: ...
    @overload
    def sendto(self, bytes: ReadableBuffer, flags: int, address: socket._Address) -> int: ...
    def send(self, bytes: ReadableBuffer, flags: int = 0) -> int: ...
    def recvfrom(self, bufsize: int, flags: int = 0) -> tuple[bytes, _Endpoint]: ...
    def recv(self, bufsize: int, flags: int = 0) -> bytes: ...
    def close(self) -> None: ...
    def get_proxy_sockname(self) -> _Endpoint | None: ...
    getproxysockname = get_proxy_sockname
    def get_proxy_peername(self) -> _Endpoint | None: ...
    getproxypeername = get_proxy_peername
    def get_peername(self) -> _Endpoint | None: ...
    getpeername = get_peername
    @set_self_blocking
    def connect(self, dest_pair: _Endpoint, catch_errors: bool | None = None) -> None: ...  # type: ignore[override]
    @set_self_blocking
    def connect_ex(self, dest_pair: _Endpoint) -> int: ...  # type: ignore[override]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol messages for describing input data Examples for machine learning
model training or inference.
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message
import tensorflow.core.example.feature_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Example(google.protobuf.message.Message):
    """An Example is a mostly-normalized data format for storing data for
    training and inference.  It contains a key-value store (features); where
    each key (string) maps to a Feature message (which is oneof packed BytesList,
    FloatList, or Int64List).  This flexible and compact format allows the
    storage of large amounts of typed data, but requires that the data shape
    and use be determined by the configuration files and parsers that are used to
    read and write this format.  That is, the Example is mostly *not* a
    self-describing format.  In TensorFlow, Examples are read in row-major
    format, so any configuration that describes data with rank-2 or above
    should keep this in mind. If you flatten a matrix into a FloatList it should
    be stored as [ row 0 ... row 1 ... row M-1 ]

    An Example for a movie recommendation application:
      features {
        feature {
          key: "age"
          value { float_list {
            value: 29.0
          }}
        }
        feature {
          key: "movie"
          value { bytes_list {
            value: "The Shawshank Redemption"
            value: "Fight Club"
          }}
        }
        feature {
          key: "movie_ratings"
          value { float_list {
            value: 9.0
            value: 9.7
          }}
        }
        feature {
          key: "suggestion"
          value { bytes_list {
            value: "Inception"
          }}
        }
        # Note that this feature exists to be used as a label in training.
        # E.g., if training a logistic regression model to predict purchase
        # probability in our learning tool we would set the label feature to
        # "suggestion_purchased".
        feature {
          key: "suggestion_purchased"
          value { float_list {
            value: 1.0
          }}
        }
        # Similar to "suggestion_purchased" above this feature exists to be used
        # as a label in training.
        # E.g., if training a linear regression model to predict purchase
        # price in our learning tool we would set the label feature to
        # "purchase_price".
        feature {
          key: "purchase_price"
          value { float_list {
            value: 9.99
          }}
        }
     }

    A conformant Example data set obeys the following conventions:
      - If a Feature K exists in one example with data type T, it must be of
          type T in all other examples when present. It may be omitted.
      - The number of instances of Feature K list data may vary across examples,
          depending on the requirements of the model.
      - If a Feature K doesn't exist in an example, a K-specific default will be
          used, if configured.
      - If a Feature K exists in an example but contains no items, the intent
          is considered to be an empty tensor and no default will be used.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURES_FIELD_NUMBER: builtins.int
    @property
    def features(self) -> tensorflow.core.example.feature_pb2.Features: ...
    def __init__(self, *, features: tensorflow.core.example.feature_pb2.Features | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["features", b"features"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["features", b"features"]) -> None: ...

global___Example = Example

@typing.final
class SequenceExample(google.protobuf.message.Message):
    """A SequenceExample is an Example representing one or more sequences, and
    some context.  The context contains features which apply to the entire
    example. The feature_lists contain a key, value map where each key is
    associated with a repeated set of Features (a FeatureList).
    A FeatureList thus represents the values of a feature identified by its key
    over time / frames.

    Below is a SequenceExample for a movie recommendation application recording a
    sequence of ratings by a user. The time-independent features ("locale",
    "age", "favorites") describing the user are part of the context. The sequence
    of movies the user rated are part of the feature_lists. For each movie in the
    sequence we have information on its name and actors and the user's rating.
    This information is recorded in three separate feature_list(s).
    In the example below there are only two movies. All three feature_list(s),
    namely "movie_ratings", "movie_names", and "actors" have a feature value for
    both movies. Note, that "actors" is itself a bytes_list with multiple
    strings per movie.

    context: {
      feature: {
        key  : "locale"
        value: {
          bytes_list: {
            value: [ "pt_BR" ]
          }
        }
      }
      feature: {
        key  : "age"
        value: {
          float_list: {
            value: [ 19.0 ]
          }
        }
      }
      feature: {
        key  : "favorites"
        value: {
          bytes_list: {
            value: [ "Majesty Rose", "Savannah Outen", "One Direction" ]
          }
        }
      }
    }
    feature_lists: {
      feature_list: {
        key  : "movie_ratings"
        value: {
          feature: {
            float_list: {
              value: [ 4.5 ]
            }
          }
          feature: {
            float_list: {
              value: [ 5.0 ]
            }
          }
        }
      }
      feature_list: {
        key  : "movie_names"
        value: {
          feature: {
            bytes_list: {
              value: [ "The Shawshank Redemption" ]
            }
          }
          feature: {
            bytes_list: {
              value: [ "Fight Club" ]
            }
          }
        }
      }
      feature_list: {
        key  : "actors"
        value: {
          feature: {
            bytes_list: {
              value: [ "Tim Robbins", "Morgan Freeman" ]
            }
          }
          feature: {
            bytes_list: {
              value: [ "Brad Pitt", "Edward Norton", "Helena Bonham Carter" ]
            }
          }
        }
      }
    }

    A conformant SequenceExample data set obeys the following conventions:

    Context:
      - All conformant context features K must obey the same conventions as
        a conformant Example's features (see above).
    Feature lists:
      - A FeatureList L may be missing in an example; it is up to the
        parser configuration to determine if this is allowed or considered
        an empty list (zero length).
      - If a FeatureList L exists, it may be empty (zero length).
      - If a FeatureList L is non-empty, all features within the FeatureList
        must have the same data type T. Even across SequenceExamples, the type T
        of the FeatureList identified by the same key must be the same. An entry
        without any values may serve as an empty feature.
      - If a FeatureList L is non-empty, it is up to the parser configuration
        to determine if all features within the FeatureList must
        have the same size.  The same holds for this FeatureList across multiple
        examples.
      - For sequence modeling, e.g.:
           http://colah.github.io/posts/2015-08-Understanding-LSTMs/
           https://github.com/tensorflow/nmt
        the feature lists represent a sequence of frames.
        In this scenario, all FeatureLists in a SequenceExample have the same
        number of Feature messages, so that the ith element in each FeatureList
        is part of the ith frame (or time step).
    Examples of conformant and non-conformant examples' FeatureLists:

    Conformant FeatureLists:
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.5 ] } }
                  feature: { float_list: { value: [ 5.0 ] } } }
       } }

    Non-conformant FeatureLists (mismatched types):
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.5 ] } }
                  feature: { int64_list: { value: [ 5 ] } } }
       } }

    Conditionally conformant FeatureLists, the parser configuration determines
    if the feature sizes must match:
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.5 ] } }
                  feature: { float_list: { value: [ 5.0, 6.0 ] } } }
       } }

    Conformant pair of SequenceExample
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.5 ] } }
                  feature: { float_list: { value: [ 5.0 ] } } }
       } }
    and:
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.5 ] } }
                  feature: { float_list: { value: [ 5.0 ] } }
                  feature: { float_list: { value: [ 2.0 ] } } }
       } }

    Conformant pair of SequenceExample
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.5 ] } }
                  feature: { float_list: { value: [ 5.0 ] } } }
       } }
    and:
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { }
       } }

    Conditionally conformant pair of SequenceExample, the parser configuration
    determines if the second feature_lists is consistent (zero-length) or
    invalid (missing "movie_ratings"):
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.5 ] } }
                  feature: { float_list: { value: [ 5.0 ] } } }
       } }
    and:
       feature_lists: { }

    Non-conformant pair of SequenceExample (mismatched types)
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.5 ] } }
                  feature: { float_list: { value: [ 5.0 ] } } }
       } }
    and:
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { int64_list: { value: [ 4 ] } }
                  feature: { int64_list: { value: [ 5 ] } }
                  feature: { int64_list: { value: [ 2 ] } } }
       } }

    Conditionally conformant pair of SequenceExample; the parser configuration
    determines if the feature sizes must match:
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.5 ] } }
                  feature: { float_list: { value: [ 5.0 ] } } }
       } }
    and:
       feature_lists: { feature_list: {
         key: "movie_ratings"
         value: { feature: { float_list: { value: [ 4.0 ] } }
                  feature: { float_list: { value: [ 5.0, 3.0 ] } }
       } }
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTEXT_FIELD_NUMBER: builtins.int
    FEATURE_LISTS_FIELD_NUMBER: builtins.int
    @property
    def context(self) -> tensorflow.core.example.feature_pb2.Features: ...
    @property
    def feature_lists(self) -> tensorflow.core.example.feature_pb2.FeatureLists: ...
    def __init__(
        self,
        *,
        context: tensorflow.core.example.feature_pb2.Features | None = ...,
        feature_lists: tensorflow.core.example.feature_pb2.FeatureLists | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["context", b"context", "feature_lists", b"feature_lists"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["context", b"context", "feature_lists", b"feature_lists"]) -> None: ...

global___SequenceExample = SequenceExample



from typing import Final, Literal, overload

TROPICALYEAR: Final[float]

@overload
def amod(a: float, b: int) -> int: ...
@overload
def amod(a: float, b: float) -> float: ...
def jwday(j: float) -> int: ...
@overload
def weekday_before(weekday: int, jd: int) -> int: ...
@overload
def weekday_before(weekday: int, jd: float) -> float: ...
@overload
def search_weekday(weekday: int, jd: int, direction: Literal[-1, 1], offset: int) -> int: ...
@overload
def search_weekday(weekday: int, jd: float, direction: Literal[-1, 1], offset: int) -> float: ...
@overload
def nearest_weekday(weekday: int, jd: int) -> int: ...
@overload
def nearest_weekday(weekday: int, jd: float) -> float: ...
@overload
def next_weekday(weekday: int, jd: int) -> int: ...
@overload
def next_weekday(weekday: int, jd: float) -> float: ...
@overload
def next_or_current_weekday(weekday: int, jd: int) -> int: ...
@overload
def next_or_current_weekday(weekday: int, jd: float) -> float: ...
@overload
def previous_weekday(weekday: int, jd: int) -> int: ...
@overload
def previous_weekday(weekday: int, jd: float) -> float: ...
@overload
def previous_or_current_weekday(weekday: int, jd: int) -> int: ...
@overload
def previous_or_current_weekday(weekday: int, jd: float) -> float: ...
@overload
def n_weeks(weekday: int, jd: int, nthweek: int) -> int: ...
@overload
def n_weeks(weekday: int, jd: float, nthweek: int) -> float: ...
def monthcalendarhelper(start_weekday: int, month_length: int) -> list[list[int | None]]: ...
def nth_day_of_month(n: int, weekday: int, month: int, year: int) -> tuple[int, int, int]: ...



from array import array
from collections.abc import Iterator
from typing import Any, Generic, Literal, NoReturn, overload
from typing_extensions import Self, TypeVar, deprecated

import numpy as np
from numpy.typing import NDArray

from .._typing import ArrayLikeSeq, GeoArray, GeoT, OptGeoArrayLike, OptGeoArrayLikeSeq
from ..constructive import BufferCapStyle, BufferJoinStyle
from ..coords import CoordinateSequence
from ..lib import Geometry
from .collection import GeometryCollection
from .point import Point
from .polygon import Polygon

GEOMETRY_TYPES: list[str]

@deprecated("Function 'geom_factory' is deprecated.")
def geom_factory(g: int, parent: object | None = None) -> Any: ...
def dump_coords(geom: Geometry) -> list[tuple[float, float] | list[tuple[float, float]]]: ...

class CAP_STYLE:
    round: Literal[BufferCapStyle.round]
    flat: Literal[BufferCapStyle.flat]
    square: Literal[BufferCapStyle.square]

class JOIN_STYLE:
    round: Literal[BufferJoinStyle.round]
    mitre: Literal[BufferJoinStyle.mitre]
    bevel: Literal[BufferJoinStyle.bevel]

class BaseGeometry(Geometry):
    __slots__: list[str] = []
    @deprecated(
        "Directly calling 'BaseGeometry()' is deprecated. To create an empty geometry, "
        "use one of the subclasses instead, for example 'GeometryCollection()'."
    )
    def __new__(self) -> GeometryCollection: ...
    def __bool__(self) -> bool: ...
    def __nonzero__(self) -> bool: ...
    def __format__(self, format_spec: str) -> str: ...
    @overload
    def __and__(self, other: Geometry) -> BaseGeometry: ...
    @overload
    def __and__(self, other: OptGeoArrayLikeSeq) -> GeoArray: ...
    @overload
    def __and__(self, other: None) -> None: ...
    @overload
    def __or__(self, other: Geometry) -> BaseGeometry: ...
    @overload
    def __or__(self, other: OptGeoArrayLikeSeq) -> GeoArray: ...
    @overload
    def __or__(self, other: None) -> None: ...
    @overload
    def __sub__(self, other: Geometry) -> BaseGeometry: ...
    @overload
    def __sub__(self, other: OptGeoArrayLikeSeq) -> GeoArray: ...
    @overload
    def __sub__(self, other: None) -> None: ...
    @overload
    def __xor__(self, other: Geometry) -> BaseGeometry: ...
    @overload
    def __xor__(self, other: OptGeoArrayLikeSeq) -> GeoArray: ...
    @overload
    def __xor__(self, other: None) -> None: ...
    def __eq__(self, other: object, /) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def coords(self) -> CoordinateSequence: ...
    @property
    def xy(self) -> tuple[array[float], array[float]]: ...
    @property
    def __geo_interface__(self) -> dict[str, Any]: ...
    @deprecated("Method 'geometryType()' is deprecated. Use attribute 'geom_type' instead.")
    def geometryType(self) -> str: ...
    @property
    @deprecated("Attribute 'type' is deprecated. Use attribute 'geom_type' instead.")
    def type(self) -> str: ...
    @property
    def wkt(self) -> str: ...
    @property
    def wkb(self) -> bytes: ...
    @property
    def wkb_hex(self) -> str: ...
    def svg(self, scale_factor: float = 1.0, **kwargs) -> str: ...
    def _repr_svg_(self) -> str: ...
    @property
    def geom_type(self) -> str: ...
    @property
    def area(self) -> float: ...
    @overload
    def distance(self, other: Geometry | None) -> float: ...
    @overload
    def distance(self, other: OptGeoArrayLikeSeq) -> NDArray[np.float64]: ...
    @overload
    def hausdorff_distance(self, other: Geometry | None) -> float: ...
    @overload
    def hausdorff_distance(self, other: OptGeoArrayLikeSeq) -> NDArray[np.float64]: ...
    @property
    def length(self) -> float: ...
    @property
    def minimum_clearance(self) -> float: ...
    @property
    def boundary(self) -> BaseMultipartGeometry | Any: ...  # is None for GeometryCollection
    @property
    def bounds(self) -> tuple[float, float, float, float]: ...
    @property
    def centroid(self) -> Point: ...
    def point_on_surface(self) -> Point: ...
    def representative_point(self) -> Point: ...
    @property
    def convex_hull(self) -> BaseGeometry: ...
    @property
    def envelope(self) -> BaseGeometry: ...
    @property
    def oriented_envelope(self) -> BaseGeometry: ...
    @property
    def minimum_rotated_rectangle(self) -> BaseGeometry: ...
    def buffer(
        self,
        distance: float,
        quad_segs: int = 16,
        cap_style: BufferCapStyle | Literal["round", "square", "flat"] = "round",
        join_style: BufferJoinStyle | Literal["round", "mitre", "bevel"] = "round",
        mitre_limit: float = 5.0,
        single_sided: bool = False,
        *,
        quadsegs: int | None = None,  # deprecated
        resolution: int | None = None,  # deprecated
    ) -> Polygon: ...
    def simplify(self, tolerance: float, preserve_topology: bool = True) -> BaseGeometry: ...
    def normalize(self) -> BaseGeometry: ...
    @overload
    def difference(self, other: Geometry, grid_size: float | None = None) -> BaseGeometry: ...
    @overload
    def difference(self, other: OptGeoArrayLikeSeq, grid_size: float | None = None) -> GeoArray: ...
    @overload
    def difference(self, other: None, grid_size: float | None = None) -> None: ...
    @overload
    def intersection(self, other: Geometry, grid_size: float | None = None) -> BaseGeometry: ...
    @overload
    def intersection(self, other: OptGeoArrayLikeSeq, grid_size: float | None = None) -> GeoArray: ...
    @overload
    def intersection(self, other: None, grid_size: float | None = None) -> None: ...
    @overload
    def symmetric_difference(self, other: Geometry, grid_size: float | None = None) -> BaseGeometry: ...
    @overload
    def symmetric_difference(self, other: OptGeoArrayLikeSeq, grid_size: float | None = None) -> GeoArray: ...
    @overload
    def symmetric_difference(self, other: None, grid_size: float | None = None) -> None: ...
    @overload
    def union(self, other: Geometry, grid_size: float | None = None) -> BaseGeometry: ...
    @overload
    def union(self, other: OptGeoArrayLikeSeq, grid_size: float | None = None) -> GeoArray: ...
    @overload
    def union(self, other: None, grid_size: float | None = None) -> None: ...
    @property
    def has_z(self) -> bool: ...
    @property
    def has_m(self) -> bool: ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def is_ring(self) -> bool: ...
    @property
    def is_closed(self) -> bool: ...
    @property
    def is_simple(self) -> bool: ...
    @property
    def is_valid(self) -> bool: ...
    @overload
    def relate(self, other: Geometry) -> str: ...
    @overload
    def relate(self, other: OptGeoArrayLikeSeq) -> NDArray[np.str_]: ...
    @overload
    def relate(self, other: None) -> None: ...
    @overload
    def covers(self, other: Geometry | None) -> bool: ...
    @overload
    def covers(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def covered_by(self, other: Geometry | None) -> bool: ...
    @overload
    def covered_by(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def contains(self, other: Geometry | None) -> bool: ...
    @overload
    def contains(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def contains_properly(self, other: Geometry | None) -> bool: ...
    @overload
    def contains_properly(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def crosses(self, other: Geometry | None) -> bool: ...
    @overload
    def crosses(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def disjoint(self, other: Geometry | None) -> bool: ...
    @overload
    def disjoint(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def equals(self, other: Geometry | None) -> bool: ...
    @overload
    def equals(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def intersects(self, other: Geometry | None) -> bool: ...
    @overload
    def intersects(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def overlaps(self, other: Geometry | None) -> bool: ...
    @overload
    def overlaps(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def touches(self, other: Geometry | None) -> bool: ...
    @overload
    def touches(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def within(self, other: Geometry | None) -> bool: ...
    @overload
    def within(self, other: OptGeoArrayLikeSeq) -> NDArray[np.bool_]: ...
    @overload
    def dwithin(self, other: Geometry | None, distance: float) -> bool: ...
    @overload
    def dwithin(self, other: OptGeoArrayLikeSeq, distance: float) -> NDArray[np.bool_]: ...
    @overload
    def dwithin(self, other: OptGeoArrayLike, distance: ArrayLikeSeq[float]) -> NDArray[np.bool_]: ...
    @overload
    def equals_exact(self, other: Geometry | None, tolerance: float = 0.0, *, normalize: bool = False) -> bool: ...
    @overload
    def equals_exact(
        self, other: OptGeoArrayLikeSeq, tolerance: float = 0.0, *, normalize: bool = False
    ) -> NDArray[np.bool_]: ...
    @overload
    def equals_exact(
        self, other: OptGeoArrayLike, tolerance: ArrayLikeSeq[float], *, normalize: bool = False
    ) -> NDArray[np.bool_]: ...
    @overload
    def relate_pattern(self, other: Geometry | None, pattern: str) -> bool: ...
    @overload
    def relate_pattern(self, other: OptGeoArrayLikeSeq, pattern: str) -> NDArray[np.bool_]: ...
    @overload
    def line_locate_point(self, other: Point | None, normalized: bool = False) -> float: ...
    @overload
    def line_locate_point(self, other: OptGeoArrayLikeSeq, normalized: bool = False) -> NDArray[np.float64]: ...
    @overload
    def project(self, other: Point | None, normalized: bool = False) -> float: ...
    @overload
    def project(self, other: OptGeoArrayLikeSeq, normalized: bool = False) -> NDArray[np.float64]: ...
    @overload
    def line_interpolate_point(self, distance: float, normalized: bool = False) -> Point: ...
    @overload
    def line_interpolate_point(self, distance: ArrayLikeSeq[float], normalized: bool = False) -> GeoArray: ...
    @overload
    def interpolate(self, distance: float, normalized: bool = False) -> Point: ...
    @overload
    def interpolate(self, distance: ArrayLikeSeq[float], normalized: bool = False) -> GeoArray: ...
    @overload
    def segmentize(self, max_segment_length: float) -> Self: ...
    @overload
    def segmentize(self, max_segment_length: ArrayLikeSeq[float]) -> GeoArray: ...
    def reverse(self) -> Self: ...

_GeoT_co = TypeVar("_GeoT_co", bound=Geometry, default=BaseGeometry, covariant=True)

class BaseMultipartGeometry(BaseGeometry, Generic[_GeoT_co]):
    __slots__: list[str] = []
    @property
    def coords(self) -> NoReturn: ...
    @property
    def geoms(self) -> GeometrySequence[Self]: ...
    def svg(self, scale_factor: float = 1.0, color: str | None = None) -> str: ...  # type: ignore[override]

_P_co = TypeVar("_P_co", covariant=True, bound=BaseMultipartGeometry[Geometry])

class GeometrySequence(Generic[_P_co]):
    def __init__(self, parent: _P_co) -> None: ...
    def __iter__(self: GeometrySequence[BaseMultipartGeometry[GeoT]]) -> Iterator[GeoT]: ...
    def __len__(self) -> int: ...
    @overload
    def __getitem__(self: GeometrySequence[BaseMultipartGeometry[GeoT]], key: int | np.integer[Any]) -> GeoT: ...
    @overload
    def __getitem__(self, key: slice) -> _P_co: ...

class EmptyGeometry(BaseGeometry):
    @deprecated(
        "The 'EmptyGeometry()' constructor is deprecated. Use one of the "
        "geometry subclasses instead, for example 'GeometryCollection()'."
    )
    def __new__(self) -> GeometryCollection: ...  # type: ignore[misc]



class CalendarError(Exception): ...
class UnsupportedDateType(CalendarError): ...
class ISORegistryError(CalendarError): ...
class ICalExportError(CalendarError): ...
class ICalExportRangeError(ICalExportError): ...
class ICalExportTargetPathError(ICalExportError): ...



import io
import re

from gunicorn.config import Config
from gunicorn.http.body import Body
from gunicorn.http.unreader import Unreader

from .._types import _AddressType

MAX_REQUEST_LINE: int
MAX_HEADERS: int
DEFAULT_MAX_HEADERFIELD_SIZE: int
RFC9110_5_6_2_TOKEN_SPECIALS: str
TOKEN_RE: re.Pattern[str]
METHOD_BADCHAR_RE: re.Pattern[str]
VERSION_RE: re.Pattern[str]
RFC9110_5_5_INVALID_AND_DANGEROUS: re.Pattern[str]

class Message:
    cfg: Config
    unreader: Unreader
    peer_addr: _AddressType
    remote_addr: _AddressType
    version: tuple[int, int] | None
    headers: list[tuple[str, str]]
    trailers: list[tuple[str, str]]
    body: Body | None
    scheme: str
    must_close: bool
    limit_request_fields: int
    limit_request_field_size: int
    max_buffer_headers: int

    def __init__(self, cfg: Config, unreader: Unreader, peer_addr: _AddressType) -> None: ...
    def force_close(self) -> None: ...
    def parse(self, unreader: Unreader) -> bytes: ...
    def parse_headers(self, data: bytes, from_trailer: bool = False) -> list[tuple[str, str]]: ...
    def set_body_reader(self) -> None: ...
    def should_close(self) -> bool: ...

class Request(Message):
    method: str | None
    uri: str | None
    path: str | None
    query: str | None
    fragment: str | None
    limit_request_line: int
    req_number: int
    proxy_protocol_info: dict[str, str | int] | None

    def __init__(self, cfg: Config, unreader: Unreader, peer_addr: _AddressType, req_number: int = 1) -> None: ...
    def get_data(self, unreader: Unreader, buf: io.BytesIO, stop: bool = False) -> None: ...
    def parse(self, unreader: Unreader) -> bytes: ...
    def read_line(self, unreader: Unreader, buf: io.BytesIO, limit: int = 0) -> tuple[bytes, bytes]: ...
    def proxy_protocol(self, line: str) -> bool: ...
    def proxy_protocol_access_check(self) -> None: ...
    def parse_proxy_protocol(self, line: str) -> None: ...
    def parse_request_line(self, line_bytes: bytes) -> None: ...
    def set_body_reader(self) -> None: ...



from dataclasses import dataclass

from seaborn._stats.base import Stat
from seaborn.external.kde import _BwMethodType

@dataclass
class KDE(Stat):
    bw_adjust: float = 1
    bw_method: _BwMethodType = "scott"
    common_norm: bool | list[str] = True
    common_grid: bool | list[str] = True
    gridsize: int | None = 200
    cut: float = 3
    cumulative: bool = False
    def __post_init__(self) -> None: ...



from typing import ClassVar
from typing_extensions import Self

import passlib.utils.handlers as uh

class cisco_pix(uh.HasUserContext, uh.StaticHandler):
    name: ClassVar[str]
    truncate_size: ClassVar[int]
    truncate_error: ClassVar[bool]
    truncate_verify_reject: ClassVar[bool]
    checksum_size: ClassVar[int]
    checksum_chars: ClassVar[str]

class cisco_asa(cisco_pix): ...

class cisco_type7(uh.GenericHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    min_salt_value: ClassVar[int]
    max_salt_value: ClassVar[int]
    @classmethod
    def using(cls, salt: int | None = None, **kwds): ...  # type: ignore[override]
    @classmethod
    def from_string(cls, hash) -> Self: ...  # type: ignore[override]
    salt: int
    def __init__(self, salt: int | None = None, **kwds) -> None: ...
    @classmethod
    def decode(cls, hash, encoding: str = "utf-8"): ...

__all__ = ["cisco_pix", "cisco_asa", "cisco_type7"]



from _typeshed import ConvertibleToInt, Incomplete
from collections.abc import Callable
from typing import Final, Literal
from typing_extensions import TypeAlias

from oauthlib.common import _HTTPMethod
from oauthlib.oauth2.rfc6749.tokens import OAuth2Token

_TokenPlacement: TypeAlias = Literal["auth_header", "query", "body"]

AUTH_HEADER: Final[_TokenPlacement]
URI_QUERY: Final[_TokenPlacement]
BODY: Final[_TokenPlacement]
FORM_ENC_HEADERS: Final[dict[str, str]]

class Client:
    refresh_token_key: str
    client_id: str
    default_token_placement: _TokenPlacement
    token_type: str
    access_token: str | None
    refresh_token: str | None
    mac_key: str | bytes | bytearray | None
    mac_algorithm: str | None
    token: dict[str, Incomplete]
    scope: str | set[object] | tuple[object] | list[object]
    state_generator: Callable[[], str]
    state: str | None
    redirect_url: str | None
    code: str | None
    expires_in: ConvertibleToInt | None
    code_verifier: str | None
    code_challenge: str | None
    code_challenge_method: str | None
    def __init__(
        self,
        client_id: str,
        default_token_placement: _TokenPlacement = "auth_header",
        token_type: str = "Bearer",
        access_token: str | None = None,
        refresh_token: str | None = None,
        mac_key: str | bytes | bytearray | None = None,
        mac_algorithm: str | None = None,
        token: dict[str, Incomplete] | None = None,
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        state: str | None = None,
        redirect_url: str | None = None,
        state_generator: Callable[[], str] = ...,
        code_verifier: str | None = None,
        code_challenge: str | None = None,
        code_challenge_method: str | None = None,
        **kwargs,
    ) -> None: ...
    @property
    def token_types(
        self,
    ) -> dict[
        Literal["Bearer", "MAC"],
        Callable[
            [str, str, str | None, dict[str, str] | None, str | None, Incomplete], tuple[str, dict[str, str] | None, str | None]
        ],
    ]: ...
    def prepare_request_uri(self, *args, **kwargs) -> str: ...
    def prepare_request_body(self, *args, **kwargs) -> str: ...
    def parse_request_uri_response(self, *args, **kwargs) -> dict[str, str]: ...
    def add_token(
        self,
        uri: str,
        http_method: _HTTPMethod = "GET",
        body: str | None = None,
        headers: dict[str, str] | None = None,
        token_placement: _TokenPlacement | None = None,
        **kwargs,
    ) -> tuple[str, dict[str, str] | None, str | None]: ...
    def prepare_authorization_request(
        self,
        authorization_url: str,
        state: str | None = None,
        redirect_url: str | None = None,
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        **kwargs,
    ) -> tuple[str, dict[str, str], str]: ...
    def prepare_token_request(
        self,
        token_url: str,
        authorization_response: str | None = None,
        redirect_url: str | None = None,
        state: str | None = None,
        body: str = "",
        **kwargs,
    ) -> tuple[str, dict[str, str], str]: ...
    def prepare_refresh_token_request(
        self,
        token_url: str,
        refresh_token: str | None = None,
        body: str = "",
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        **kwargs,
    ) -> tuple[str, dict[str, str], str]: ...
    def prepare_token_revocation_request(
        self,
        revocation_url: str,
        token: str,
        token_type_hint: Literal["access_token", "refresh_token"] | None = "access_token",
        body: str = "",
        callback: Callable[[Incomplete], Incomplete] | None = None,
        **kwargs,
    ): ...
    def parse_request_body_response(
        self, body: str, scope: str | set[object] | tuple[object] | list[object] | None = None, **kwargs
    ) -> OAuth2Token: ...
    def prepare_refresh_body(
        self,
        body: str = "",
        refresh_token: str | None = None,
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        **kwargs,
    ) -> str: ...
    def create_code_verifier(self, length: int) -> str: ...
    def create_code_challenge(self, code_verifier: str, code_challenge_method: str | None = None) -> str: ...
    def populate_code_attributes(self, response: dict[str, Incomplete]) -> None: ...
    def populate_token_attributes(self, response: dict[str, Incomplete]) -> None: ...



from _typeshed import Incomplete, Unused
from abc import ABC
from collections.abc import Iterable, Iterator, Mapping, Set as AbstractSet
from typing import Any, Generic, Literal, TypeVar, overload
from typing_extensions import Self

from networkx.classes.graph import Graph, _Edge, _NBunch, _Node

_D = TypeVar("_D")
_U = TypeVar("_U")

__all__ = [
    "NodeView",
    "NodeDataView",
    "EdgeView",
    "OutEdgeView",
    "InEdgeView",
    "EdgeDataView",
    "OutEdgeDataView",
    "InEdgeDataView",
    "MultiEdgeView",
    "OutMultiEdgeView",
    "InMultiEdgeView",
    "MultiEdgeDataView",
    "OutMultiEdgeDataView",
    "InMultiEdgeDataView",
    "DegreeView",
    "DiDegreeView",
    "InDegreeView",
    "OutDegreeView",
    "MultiDegreeView",
    "DiMultiDegreeView",
    "InMultiDegreeView",
    "OutMultiDegreeView",
]

class NodeView(Mapping[_Node, dict[str, Any]], AbstractSet[_Node]):
    __slots__ = ("_nodes",)
    def __init__(self, graph: Graph[_Node]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_Node]: ...
    def __getitem__(self, n: _Node) -> dict[str, Any]: ...
    def __contains__(self, n: object) -> bool: ...
    @overload
    def __call__(self, data: Literal[False] = False, default=None) -> Self: ...
    @overload
    def __call__(self, data: Literal[True] | str, default=None) -> NodeDataView[_Node]: ...
    @overload
    def data(self, data: Literal[False], default=None) -> Self: ...
    @overload
    def data(self, data: Literal[True] | str = True, default=None) -> NodeDataView[_Node]: ...

class NodeDataView(AbstractSet[_Node]):
    __slots__ = ("_nodes", "_data", "_default")
    def __init__(self, nodedict: Mapping[str, Incomplete], data: bool | str = False, default=None) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[tuple[_Node, Incomplete]]: ...  # type: ignore[override]
    def __contains__(self, n: object) -> bool: ...
    def __getitem__(self, n: _Node): ...

class DiDegreeView(Generic[_Node]):
    def __init__(self, G: Graph[_Node], nbunch: _NBunch[_Node] = None, weight: None | bool | str = None) -> None: ...
    @overload  # Use this overload first in case _Node=str, since `str` matches `Iterable[str]`
    def __call__(self, nbunch: _Node, weight: None | bool | str = None) -> int: ...  # type: ignore[overload-overlap]
    @overload
    def __call__(self, nbunch: Iterable[_Node] | None = None, weight: None | bool | str = None) -> Self: ...
    def __getitem__(self, n: _Node) -> int: ...
    def __iter__(self) -> Iterator[tuple[_Node, int]]: ...
    def __len__(self) -> int: ...

class DegreeView(DiDegreeView[_Node]): ...
class OutDegreeView(DiDegreeView[_Node]): ...
class InDegreeView(DiDegreeView[_Node]): ...
class MultiDegreeView(DiDegreeView[_Node]): ...
class DiMultiDegreeView(DiDegreeView[_Node]): ...
class InMultiDegreeView(DiDegreeView[_Node]): ...
class OutMultiDegreeView(DiDegreeView[_Node]): ...
class EdgeViewABC(ABC): ...

class OutEdgeDataView(EdgeViewABC, Generic[_Node, _D]):
    __slots__ = ("_viewer", "_nbunch", "_data", "_default", "_adjdict", "_nodes_nbrs", "_report")
    def __init__(self, viewer, nbunch: _NBunch[_Node] = None, data: bool = False, *, default=None) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_D]: ...
    def __contains__(self, e: _Edge[_Node]) -> bool: ...

class EdgeDataView(OutEdgeDataView[_Node, _D]):
    __slots__ = ()

class InEdgeDataView(OutEdgeDataView[_Node, _D]):
    __slots__ = ()

class OutMultiEdgeDataView(OutEdgeDataView[_Node, _D]):
    __slots__ = ("keys",)
    keys: bool
    def __init__(
        self, viewer, nbunch: _NBunch[_Node] = None, data: bool = False, *, default=None, keys: bool = False
    ) -> None: ...

class MultiEdgeDataView(OutEdgeDataView[_Node, _D]):
    __slots__ = ()

class InMultiEdgeDataView(OutEdgeDataView[_Node, _D]):
    __slots__ = ()

class OutEdgeView(AbstractSet[Incomplete], Mapping[Incomplete, Incomplete], EdgeViewABC, Generic[_Node]):
    __slots__ = ("_adjdict", "_graph", "_nodes_nbrs")
    def __init__(self, G: Graph[_Node]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[tuple[_Node, _Node]]: ...
    def __contains__(self, e: _Edge[_Node]) -> bool: ...  # type: ignore[override]
    def __getitem__(self, e: _Edge[_Node]) -> dict[str, Any]: ...
    dataview = OutEdgeDataView
    @overload
    def __call__(self, nbunch: None = None, data: Literal[False] = False, *, default: Unused = None) -> Self: ...  # type: ignore[overload-overlap]
    @overload
    def __call__(
        self, nbunch: _Node | Iterable[_Node], data: Literal[False] = False, *, default: None = None
    ) -> OutEdgeDataView[_Node, tuple[_Node, _Node]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: Literal[True], *, default: None = None
    ) -> OutEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: Literal[True], default: None = None
    ) -> OutEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: str, *, default: _U | None = None
    ) -> OutEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: str, default: _U | None = None
    ) -> OutEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def data(self, data: Literal[False], default: Unused = None, nbunch: None = None) -> Self: ...
    @overload
    def data(
        self, data: Literal[True] = True, default: None = None, nbunch: _NBunch[_Node] = None
    ) -> OutEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def data(
        self, data: str, default: _U | None = None, nbunch: _NBunch[_Node] = None
    ) -> OutEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...

class EdgeView(OutEdgeView[_Node]):
    __slots__ = ()
    dataview = EdgeDataView
    # Have to override parent's overloads with the proper return type based on dataview
    @overload
    def __call__(self, nbunch: None = None, data: Literal[False] = False, *, default: Unused = None) -> Self: ...  # type: ignore[overload-overlap]
    @overload
    def __call__(
        self, nbunch: _Node | Iterable[_Node], data: Literal[False] = False, *, default: None = None
    ) -> EdgeDataView[_Node, tuple[_Node, _Node]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: Literal[True], *, default: None = None
    ) -> EdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: Literal[True], default: None = None
    ) -> EdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: str, *, default: _U | None = None
    ) -> EdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: str, default: _U | None = None
    ) -> EdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def data(self, data: Literal[False], default: Unused = None, nbunch: None = None) -> Self: ...
    @overload
    def data(
        self, data: Literal[True] = True, default: None = None, nbunch: _NBunch[_Node] = None
    ) -> EdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def data(
        self, data: str, default: _U | None = None, nbunch: _NBunch[_Node] = None
    ) -> EdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...

class InEdgeView(OutEdgeView[_Node]):
    __slots__ = ()
    dataview = InEdgeDataView
    # Have to override parent's overloads with the proper return type based on dataview
    @overload
    def __call__(self, nbunch: None = None, data: Literal[False] = False, *, default: Unused = None) -> Self: ...  # type: ignore[overload-overlap]
    @overload
    def __call__(
        self, nbunch: _Node | Iterable[_Node], data: Literal[False] = False, *, default: None = None
    ) -> InEdgeDataView[_Node, tuple[_Node, _Node]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: Literal[True], *, default: None = None
    ) -> InEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: Literal[True], default: None = None
    ) -> InEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: str, *, default: _U | None = None
    ) -> InEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: str, default: _U | None = None
    ) -> InEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def data(self, data: Literal[False], default: Unused = None, nbunch: None = None) -> Self: ...
    @overload
    def data(
        self, data: Literal[True] = True, default: None = None, nbunch: _NBunch[_Node] = None
    ) -> InEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def data(
        self, data: str, default: _U | None = None, nbunch: _NBunch[_Node] = None
    ) -> InEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...

class OutMultiEdgeView(OutEdgeView[_Node]):
    __slots__ = ()
    def __iter__(self) -> Iterator[tuple[_Node, _Node, Incomplete]]: ...  # type: ignore[override]
    def __getitem__(self, e: tuple[_Node, _Node, Incomplete]) -> dict[str, Any]: ...  # type: ignore[override]
    dataview = OutMultiEdgeDataView
    @overload  # type: ignore[override]  # Has an additional `keys` keyword argument
    def __call__(  # type: ignore[overload-overlap]
        self, nbunch: None = None, data: Literal[False] = False, *, default: Unused = None, keys: Literal[True]
    ) -> Self: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, data: Literal[False] = False, *, default: None = None, keys: Literal[False] = False
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node]]: ...
    @overload
    def __call__(
        self, nbunch: _Node | Iterable[_Node], data: Literal[False] = False, *, default: None = None, keys: Literal[True]
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: Literal[True], default: None = None, keys: Literal[True]
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: Literal[True], *, default: None = None, keys: Literal[True]
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: Literal[True], default: None = None, keys: Literal[False] = False
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: str, *, default: _U | None = None, keys: Literal[False] = False
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: str, default: _U | None = None, keys: Literal[True]
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, _U]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: str, default: _U | None = None, keys: Literal[False] = False
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload  # type: ignore[override]
    def data(self, data: Literal[False], default: Unused = None, nbunch: None = None, *, keys: Literal[True]) -> Self: ...
    @overload
    def data(
        self, data: Literal[False], default: None = None, nbunch: _NBunch[_Node] = None, keys: Literal[False] = False
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node]]: ...
    @overload
    def data(
        self, data: Literal[True] = True, default: None = None, nbunch: _NBunch[_Node] = None, keys: Literal[False] = False
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def data(
        self, data: Literal[True] = True, default: None = None, nbunch: _NBunch[_Node] = None, *, keys: Literal[True]
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, dict[str, Incomplete]]]: ...
    @overload
    def data(
        self, data: str, default: _U | None = None, nbunch: _NBunch[_Node] = None, keys: Literal[False] = False
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def data(
        self, data: str, default: _U | None = None, nbunch: _NBunch[_Node] = None, *, keys: Literal[True]
    ) -> OutMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, _U]]: ...

class MultiEdgeView(OutMultiEdgeView[_Node]):
    __slots__ = ()
    dataview = MultiEdgeDataView  # type: ignore[assignment]
    # Have to override parent's overloads with the proper return type based on dataview
    @overload  # type: ignore[override]  # Has an additional `keys` keyword argument
    def __call__(  # type: ignore[overload-overlap]
        self, nbunch: None = None, data: Literal[False] = False, *, default: Unused = None, keys: Literal[True]
    ) -> Self: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, data: Literal[False] = False, *, default: None = None, keys: Literal[False] = False
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node]]: ...
    @overload
    def __call__(
        self, nbunch: _Node | Iterable[_Node], data: Literal[False] = False, *, default: None = None, keys: Literal[True]
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: Literal[True], default: None = None, keys: Literal[True]
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: Literal[True], *, default: None = None, keys: Literal[True]
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: Literal[True], default: None = None, keys: Literal[False] = False
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: str, *, default: _U | None = None, keys: Literal[False] = False
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: str, default: _U | None = None, keys: Literal[True]
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, _U]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: str, default: _U | None = None, keys: Literal[False] = False
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload  # type: ignore[override]
    def data(self, data: Literal[False], default: Unused = None, nbunch: None = None, *, keys: Literal[True]) -> Self: ...
    @overload
    def data(
        self, data: Literal[False], default: None = None, nbunch: _NBunch[_Node] = None, keys: Literal[False] = False
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node]]: ...
    @overload
    def data(
        self, data: Literal[True] = True, default: None = None, nbunch: _NBunch[_Node] = None, keys: Literal[False] = False
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def data(
        self, data: Literal[True] = True, default: None = None, nbunch: _NBunch[_Node] = None, *, keys: Literal[True]
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, dict[str, Incomplete]]]: ...
    @overload
    def data(
        self, data: str, default: _U | None = None, nbunch: _NBunch[_Node] = None, keys: Literal[False] = False
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def data(
        self, data: str, default: _U | None = None, nbunch: _NBunch[_Node] = None, *, keys: Literal[True]
    ) -> MultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, _U]]: ...

class InMultiEdgeView(OutMultiEdgeView[_Node]):
    __slots__ = ()
    dataview = InMultiEdgeDataView  # type: ignore[assignment]
    # Have to override parent's overloads with the proper return type based on dataview
    @overload  # type: ignore[override]
    def __call__(  # type: ignore[overload-overlap]
        self, nbunch: None = None, data: Literal[False] = False, *, default: Unused = None, keys: Literal[True]
    ) -> Self: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, data: Literal[False] = False, *, default: None = None, keys: Literal[False] = False
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node]]: ...
    @overload
    def __call__(
        self, nbunch: _Node | Iterable[_Node], data: Literal[False] = False, *, default: None = None, keys: Literal[True]
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: Literal[True], default: None = None, keys: Literal[True]
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: Literal[True], *, default: None = None, keys: Literal[True]
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: Literal[True], default: None = None, keys: Literal[False] = False
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node], data: str, *, default: _U | None = None, keys: Literal[False] = False
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: str, default: _U | None = None, keys: Literal[True]
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, _U]]: ...
    @overload
    def __call__(
        self, nbunch: _NBunch[_Node] = None, *, data: str, default: _U | None = None, keys: Literal[False] = False
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload  # type: ignore[override]
    def data(self, data: Literal[False], default: Unused = None, nbunch: None = None, *, keys: Literal[True]) -> Self: ...
    @overload
    def data(
        self, data: Literal[False], default: None = None, nbunch: _NBunch[_Node] = None, keys: Literal[False] = False
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node]]: ...
    @overload
    def data(
        self, data: Literal[True] = True, default: None = None, nbunch: _NBunch[_Node] = None, keys: Literal[False] = False
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, dict[str, Incomplete]]]: ...
    @overload
    def data(
        self, data: Literal[True] = True, default: None = None, nbunch: _NBunch[_Node] = None, *, keys: Literal[True]
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, dict[str, Incomplete]]]: ...
    @overload
    def data(
        self, data: str, default: _U | None = None, nbunch: _NBunch[_Node] = None, keys: Literal[False] = False
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, _U]]: ...
    @overload
    def data(
        self, data: str, default: _U | None = None, nbunch: _NBunch[_Node] = None, *, keys: Literal[True]
    ) -> InMultiEdgeDataView[_Node, tuple[_Node, _Node, Incomplete, _U]]: ...



from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["number_connected_components", "connected_components", "is_connected", "node_connected_component"]

@_dispatchable
def connected_components(G: Graph[_Node]) -> Generator[set[_Node]]: ...
@_dispatchable
def number_connected_components(G: Graph[_Node]) -> int: ...
@_dispatchable
def is_connected(G: Graph[_Node]) -> bool: ...
@_dispatchable
def node_connected_component(G: Graph[_Node], n: _Node) -> set[_Node]: ...



from _typeshed import Incomplete
from typing import Any

# Formatter[str] from types-pygments
class _Formatter:
    name: Any
    aliases: Any
    filenames: Any
    unicodeoutput: bool
    style: Any
    full: Any
    title: Any
    encoding: Any
    options: Any
    def __init__(self, *, encoding: None = None, outencoding: None = None, **options) -> None: ...
    def get_style_defs(self, arg: str = ""): ...
    def format(self, tokensource, outfile): ...

class OdtPygmentsFormatter(_Formatter):
    rststyle_function: Incomplete
    escape_function: Incomplete
    def __init__(self, rststyle_function, escape_function) -> None: ...
    def rststyle(self, name, parameters=()): ...
    def get_style_defs(self, arg: str = ""): ...
    def format(self, tokensource, outfile): ...

class OdtPygmentsProgFormatter(OdtPygmentsFormatter):
    def format(self, tokensource, outfile) -> None: ...

class OdtPygmentsLaTeXFormatter(OdtPygmentsFormatter):
    def format(self, tokensource, outfile) -> None: ...



import functools
import logging
from pygls.constants import ATTR_FEATURE_TYPE
from pygls.feature_manager import assign_help_attrs


logger = logging.getLogger(__name__)


def call_user_feature(base_func, method_name):
    """Wraps generic LSP features and calls user registered feature
    immediately after it.
    """

    @functools.wraps(base_func)
    def decorator(self, *args, **kwargs):
        ret_val = base_func(self, *args, **kwargs)

        try:
            user_func = self.fm.features[method_name]
            self._execute_notification(user_func, *args, **kwargs)
        except KeyError:
            pass
        except Exception:
            logger.exception(
                'Failed to handle user defined notification "%s": %s', method_name, args
            )

        return ret_val

    return decorator


class LSPMeta(type):
    """Wraps LSP built-in features (`lsp_` naming convention).

    Built-in features cannot be overridden but user defined features with
    the same LSP name will be called after them.
    """

    def __new__(mcs, cls_name, cls_bases, cls):
        for attr_name, attr_val in cls.items():
            if callable(attr_val) and hasattr(attr_val, "method_name"):
                method_name = attr_val.method_name
                wrapped = call_user_feature(attr_val, method_name)
                assign_help_attrs(wrapped, method_name, ATTR_FEATURE_TYPE)
                cls[attr_name] = wrapped

                logger.debug('Added decorator for lsp method: "%s"', attr_name)

        return super().__new__(mcs, cls_name, cls_bases, cls)



from datetime import date
from typing_extensions import Self

__tracebackhide__: bool

class DateMixin:
    def is_before(self, other: date) -> Self: ...
    def is_after(self, other: date) -> Self: ...
    def is_equal_to_ignoring_milliseconds(self, other: date) -> Self: ...
    def is_equal_to_ignoring_seconds(self, other: date) -> Self: ...
    def is_equal_to_ignoring_time(self, other: date) -> Self: ...



"""Jac Compiler - handles all compilation operations separate from program state."""

from __future__ import annotations

import ast as py_ast
import marshal
import os
import types
from threading import Event
from typing import TYPE_CHECKING

import jaclang.pycore.unitree as uni
from jaclang.pycore.bccache import (
    BytecodeCache,
    CacheKey,
    get_bytecode_cache,
)
from jaclang.pycore.helpers import read_file_with_encoding
from jaclang.pycore.jac_parser import JacParser
from jaclang.pycore.passes import (
    DeclImplMatchPass,
    JacAnnexPass,
    PyastGenPass,
    PyBytecodeGenPass,
    SemanticAnalysisPass,
    SymTabBuildPass,
    Transform,
)
from jaclang.pycore.tsparser import TypeScriptParser

if TYPE_CHECKING:
    from jaclang.pycore.program import JacProgram


def get_symtab_ir_sched() -> list[type[Transform[uni.Module, uni.Module]]]:
    """Symbol table build schedule."""
    return [SymTabBuildPass, DeclImplMatchPass]


def get_ir_gen_sched() -> list[type[Transform[uni.Module, uni.Module]]]:
    """Full IR generation schedule."""
    from jaclang.compiler.passes.main import CFGBuildPass, SemDefMatchPass

    return [
        SymTabBuildPass,
        DeclImplMatchPass,
        SemanticAnalysisPass,
        SemDefMatchPass,
        CFGBuildPass,
    ]


def get_type_check_sched() -> list[type[Transform[uni.Module, uni.Module]]]:
    """Type checking schedule."""
    from jaclang.compiler.passes.main import TypeCheckPass

    return [TypeCheckPass]


def get_py_code_gen() -> list[type[Transform[uni.Module, uni.Module]]]:
    """Full Python code generation schedule."""
    from jaclang.compiler.passes.ecmascript import EsastGenPass
    from jaclang.compiler.passes.main import PyJacAstLinkPass

    return [EsastGenPass, PyastGenPass, PyJacAstLinkPass, PyBytecodeGenPass]


def get_minimal_ir_gen_sched() -> list[type[Transform[uni.Module, uni.Module]]]:
    """Minimal IR schedule (no CFG) for bootstrap modules."""
    return [SymTabBuildPass, DeclImplMatchPass, SemanticAnalysisPass]


def get_minimal_py_code_gen() -> list[type[Transform[uni.Module, uni.Module]]]:
    """Minimal codegen (bytecode only, no JS) for bootstrap modules."""
    return [PyastGenPass, PyBytecodeGenPass]


def get_format_sched(
    auto_lint: bool = False,
) -> list[type[Transform[uni.Module, uni.Module]]]:
    """Format schedule. If auto_lint=True, includes auto-linting pass."""
    from jaclang.compiler.passes.tool.comment_injection_pass import (
        CommentInjectionPass,
    )
    from jaclang.compiler.passes.tool.doc_ir_gen_pass import DocIRGenPass
    from jaclang.compiler.passes.tool.jac_auto_lint_pass import JacAutoLintPass
    from jaclang.compiler.passes.tool.jac_formatter_pass import JacFormatPass
    from jaclang.pycore.passes.annex_pass import JacAnnexPass

    if auto_lint:
        return [
            JacAnnexPass,
            JacAutoLintPass,
            DocIRGenPass,
            CommentInjectionPass,
            JacFormatPass,
        ]
    else:
        return [
            DocIRGenPass,
            CommentInjectionPass,
            JacFormatPass,
        ]


class JacCompiler:
    """Jac Compiler singleton.

    Maintains separate module hub for jaclang.* modules (internal_program)
    vs user modules (target_program passed to methods).
    """

    _jaclang_root: str | None = None

    def __init__(self, bytecode_cache: BytecodeCache | None = None) -> None:
        """Initialize with optional custom bytecode cache."""
        self._bytecode_cache: BytecodeCache = bytecode_cache or get_bytecode_cache()
        self._internal_program: JacProgram | None = None

    @property
    def internal_program(self) -> JacProgram:
        """Module hub for jaclang.* modules (lazily initialized)."""
        if self._internal_program is None:
            from jaclang.pycore.program import JacProgram

            self._internal_program = JacProgram()
        return self._internal_program

    @classmethod
    def _get_jaclang_root(cls) -> str:
        """Get the jaclang package root path."""
        if cls._jaclang_root is None:
            cls._jaclang_root = os.path.dirname(os.path.dirname(__file__))
        return cls._jaclang_root

    def _resolve_program(
        self, file_path: str, target_program: JacProgram
    ) -> JacProgram:
        """Route jaclang.* files to internal_program, others to target_program."""
        normalized_path = os.path.normpath(file_path)
        jaclang_root = self._get_jaclang_root()
        if normalized_path.startswith(jaclang_root + os.sep):
            return self.internal_program
        return target_program

    def get_bytecode(
        self, full_target: str, target_program: JacProgram, minimal: bool = False
    ) -> types.CodeType | None:
        """Get bytecode using 3-tier cache: in-memory -> disk -> compile."""
        actual_program = self._resolve_program(full_target, target_program)

        # Tier 1: In-memory cache
        if (
            full_target in actual_program.mod.hub
            and actual_program.mod.hub[full_target].gen.py_bytecode
        ):
            codeobj = actual_program.mod.hub[full_target].gen.py_bytecode
            return marshal.loads(codeobj) if isinstance(codeobj, bytes) else None

        # Tier 2: Disk cache
        cache_key = CacheKey.for_source(full_target, minimal)
        cached_code = self._bytecode_cache.get(cache_key)
        if cached_code is not None:
            return cached_code

        # Tier 3: Compile
        result = self.compile(
            file_path=full_target, target_program=actual_program, minimal=minimal
        )
        if result.gen.py_bytecode:
            self._bytecode_cache.put(cache_key, result.gen.py_bytecode)
            return marshal.loads(result.gen.py_bytecode)
        return None

    def parse_str(
        self,
        source_str: str,
        file_path: str,
        target_program: JacProgram,
        cancel_token: Event | None = None,
    ) -> uni.Module:
        """Parse source string into AST module."""
        had_error = False
        if file_path.endswith(".py") or file_path.endswith(".pyi"):
            from jaclang.compiler.passes.main import PyastBuildPass

            parsed_ast = py_ast.parse(source_str)
            py_ast_ret = PyastBuildPass(
                ir_in=uni.PythonModuleAst(
                    parsed_ast,
                    orig_src=uni.Source(source_str, mod_path=file_path),
                ),
                prog=target_program,
                cancel_token=cancel_token,
            )
            had_error = len(py_ast_ret.errors_had) > 0
            mod = py_ast_ret.ir_out
        elif file_path.endswith((".js", ".ts", ".jsx", ".tsx")):
            source = uni.Source(source_str, mod_path=file_path)
            ts_ast_ret = TypeScriptParser(
                root_ir=source, prog=target_program, cancel_token=cancel_token
            )
            had_error = len(ts_ast_ret.errors_had) > 0
            mod = ts_ast_ret.ir_out
        else:
            source = uni.Source(source_str, mod_path=file_path)
            jac_ast_ret: Transform[uni.Source, uni.Module] = JacParser(
                root_ir=source, prog=target_program
            )
            had_error = len(jac_ast_ret.errors_had) > 0
            mod = jac_ast_ret.ir_out
        if had_error:
            return mod
        if target_program.mod.main.stub_only:
            target_program.mod = uni.ProgramModule(mod)
        target_program.mod.hub[mod.loc.mod_path] = mod
        JacAnnexPass(ir_in=mod, prog=target_program)
        return mod

    def compile(
        self,
        file_path: str,
        target_program: JacProgram,
        use_str: str | None = None,
        no_cgen: bool = False,
        type_check: bool = False,
        symtab_ir_only: bool = False,
        minimal: bool = False,
        cancel_token: Event | None = None,
    ) -> uni.Module:
        """Compile a Jac file into module AST."""
        actual_program = self._resolve_program(file_path, target_program)

        keep_str = use_str or read_file_with_encoding(file_path)
        mod_targ = self.parse_str(
            keep_str, file_path, actual_program, cancel_token=cancel_token
        )
        if symtab_ir_only:
            self.run_schedule(
                mod=mod_targ,
                target_program=actual_program,
                passes=get_symtab_ir_sched(),
                cancel_token=cancel_token,
            )
        elif minimal:
            self.run_schedule(
                mod=mod_targ,
                target_program=actual_program,
                passes=get_minimal_ir_gen_sched(),
                cancel_token=cancel_token,
            )
        else:
            self.run_schedule(
                mod=mod_targ,
                target_program=actual_program,
                passes=get_ir_gen_sched(),
                cancel_token=cancel_token,
            )
        if type_check and not minimal:
            self.run_schedule(
                mod=mod_targ,
                target_program=actual_program,
                passes=get_type_check_sched(),
                cancel_token=cancel_token,
            )
        if (not mod_targ.has_syntax_errors) and (not no_cgen):
            codegen_sched = get_minimal_py_code_gen() if minimal else get_py_code_gen()
            self.run_schedule(
                mod=mod_targ,
                target_program=actual_program,
                passes=codegen_sched,
                cancel_token=cancel_token,
            )
        return mod_targ

    def build(
        self,
        file_path: str,
        target_program: JacProgram,
        use_str: str | None = None,
        type_check: bool = False,
    ) -> uni.Module:
        """Build a Jac file with import dependency resolution."""

        actual_program = self._resolve_program(file_path, target_program)

        mod_targ = self.compile(
            file_path, actual_program, use_str, type_check=type_check
        )
        SemanticAnalysisPass(ir_in=mod_targ, prog=actual_program)
        return mod_targ

    def run_schedule(
        self,
        mod: uni.Module,
        target_program: JacProgram,
        passes: list[type[Transform[uni.Module, uni.Module]]],
        cancel_token: Event | None = None,
    ) -> None:
        """Run a list of passes on a module."""
        for current_pass in passes:
            current_pass(ir_in=mod, prog=target_program, cancel_token=cancel_token)  # type: ignore

    @staticmethod
    def jac_file_formatter(file_path: str, auto_lint: bool = False) -> JacProgram:
        """Format a Jac file."""
        from jaclang.pycore.program import JacProgram

        prog = JacProgram()
        source_str = read_file_with_encoding(file_path)
        source = uni.Source(source_str, mod_path=file_path)
        parser_pass = JacParser(root_ir=source, prog=prog)
        current_mod = parser_pass.ir_out
        for pass_cls in get_format_sched(auto_lint=auto_lint):
            current_mod = pass_cls(ir_in=current_mod, prog=prog).ir_out
        prog.mod = uni.ProgramModule(current_mod)
        return prog

    @staticmethod
    def jac_str_formatter(
        source_str: str, file_path: str, auto_lint: bool = False
    ) -> JacProgram:
        """Format a Jac source string."""
        from jaclang.pycore.program import JacProgram

        prog = JacProgram()
        source = uni.Source(source_str, mod_path=file_path)
        parser_pass = JacParser(root_ir=source, prog=prog)
        current_mod = parser_pass.ir_out
        for pass_cls in get_format_sched(auto_lint=auto_lint):
            current_mod = pass_cls(ir_in=current_mod, prog=prog).ir_out
        prog.mod = uni.ProgramModule(current_mod)
        return prog



import re
from collections.abc import Generator, Mapping
from typing import Any, TextIO, TypeVar

_AT = TypeVar("_AT", int, str, re.Match[str], None)

class _MatchParser:
    def __init__(self, string: str) -> None: ...
    def match(self, r: re.Pattern[str]) -> re.Match[str] | int | None: ...
    def advance(self, by: _AT) -> _AT: ...
    def consume(self, r: re.Pattern[str]) -> re.Match[str]: ...
    def child(self) -> _MatchChildParser: ...

class _MatchChildParser(_MatchParser):
    def __init__(self, parent: _MatchParser) -> None: ...
    def commit(self) -> _MatchParser: ...

class ParseError(Exception):
    def __init__(self, parser: _MatchParser) -> None: ...

class Block:
    def __init__(self, **kwargs: Any) -> None: ...  # Abstract. Accepts arbitrary keyword arguments.
    @classmethod
    def parse(cls, parser: _MatchParser) -> Block: ...
    def write_into(self, stream: TextIO) -> None: ...

class HeaderBlock(Block): ...

class Magic(HeaderBlock):
    @classmethod
    def parse(cls, parser: _MatchParser) -> Magic: ...
    def write_into(self, stream: TextIO) -> None: ...

class StyleBlock(HeaderBlock): ...
class RegionBlock(HeaderBlock): ...
class CommentBlock(Block): ...

class CueBlock(Block):
    @classmethod
    def parse(cls, parser: _MatchParser) -> CueBlock: ...
    def write_into(self, stream: TextIO) -> None: ...
    @property
    def as_json(self) -> dict[str, Any]: ...
    def __eq__(self, other: object) -> bool: ...
    @classmethod
    def from_json(cls, json: Mapping[str, Any]) -> CueBlock: ...
    def hinges(self, other: Block) -> bool: ...

def parse_fragment(frag_content: bytes) -> Generator[Block]: ...



from typing import Any, ClassVar, Literal, overload

import passlib.utils.handlers as uh

class lmhash(uh.TruncateMixin, uh.HasEncodingContext, uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    truncate_size: ClassVar[int]
    @classmethod
    def raw(cls, secret, encoding=None): ...

class nthash(uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    @classmethod
    def raw(cls, secret): ...
    @overload
    @classmethod
    def raw_nthash(cls, secret: str | bytes, hex: Literal[True]) -> str: ...
    @overload
    @classmethod
    def raw_nthash(cls, secret: str | bytes, hex: Literal[False] = False) -> bytes: ...
    @overload
    @classmethod
    def raw_nthash(cls, secret: str | bytes, hex: bool = False) -> str | bytes: ...

bsd_nthash: Any

class msdcc(uh.HasUserContext, uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    @classmethod
    def raw(cls, secret, user): ...

class msdcc2(uh.HasUserContext, uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]
    @classmethod
    def raw(cls, secret, user): ...

__all__ = ["lmhash", "nthash", "bsd_nthash", "msdcc", "msdcc2"]



from abc import abstractmethod
from logging import Logger
from typing import Generic, TypeVar

from .amqp_object import AMQPObject, Method as AMQPMethod
from .spec import BasicProperties

_M = TypeVar("_M", bound=AMQPMethod)

LOGGER: Logger

class Frame(AMQPObject):
    frame_type: int
    channel_number: int
    def __init__(self, frame_type: int, channel_number: int) -> None: ...
    @abstractmethod
    def marshal(self) -> bytes: ...

class Method(Frame, Generic[_M]):
    method: _M
    def __init__(self, channel_number: int, method: _M) -> None: ...
    def marshal(self) -> bytes: ...

class Header(Frame):
    body_size: int
    properties: BasicProperties
    def __init__(self, channel_number: int, body_size: int, props: BasicProperties) -> None: ...
    def marshal(self) -> bytes: ...

class Body(Frame):
    fragment: bytes
    def __init__(self, channel_number: int, fragment: bytes) -> None: ...
    def marshal(self) -> bytes: ...

class Heartbeat(Frame):
    def __init__(self) -> None: ...
    def marshal(self) -> bytes: ...

class ProtocolHeader(AMQPObject):
    frame_type: int
    major: int
    minor: int
    revision: int
    def __init__(self, major: int | None = None, minor: int | None = None, revision: int | None = None) -> None: ...
    def marshal(self) -> bytes: ...

def decode_frame(data_in: bytes) -> tuple[int, Frame | None]: ...



from typing import Final

EPOCH: Final = 2394646.5
EPOCH_GREGORIAN_YEAR: Final = 1844
TEHRAN: Final = 51.4215, 35.6944
WEEKDAYS: Final = ("Jamál", "Kamál", "Fidál", "Idál", "Istijlál", "Istiqlál", "Jalál")
MONTHS: Final[tuple[str, ...]]
ENGLISH_MONTHS: Final[tuple[str, ...]]
BAHA: Final = 1
JALAL: Final = 2
JAMAL: Final = 3
AZAMAT: Final = 4
NUR: Final = 5
RAHMAT: Final = 6
KALIMAT: Final = 7
KAMAL: Final = 8
ASMA: Final = 9
IZZAT: Final = 10
MASHIYYAT: Final = 11
ILM: Final = 12
QUDRAT: Final = 13
QAWL: Final = 14
MASAIL: Final = 15
SHARAF: Final = 16
SULTAN: Final = 17
MULK: Final = 18
AYYAMIHA: Final = 19
ALA: Final = 20

def gregorian_nawruz(year: int) -> tuple[int, int]: ...
def to_jd(year: int, month: int, day: int) -> float: ...
def from_jd(jd: float) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def to_gregorian(year: int, month: int, day: int) -> tuple[int, int, int]: ...
def month_length(year: int, month: int) -> int: ...
def monthcalendar(year: int, month: int) -> list[list[int | None]]: ...
def format(year: int, month: int, day: int, lang: str | None = None) -> str: ...



from _typeshed import ConvertibleToInt, Unused
from collections.abc import Generator
from typing import Literal, overload

from openpyxl.descriptors import Strict
from openpyxl.descriptors.base import MinMax, String
from openpyxl.workbook.child import _WorkbookChild
from openpyxl.worksheet._read_only import ReadOnlyWorksheet

class DummyWorksheet:
    title: str
    def __init__(self, title: str) -> None: ...

class Reference(Strict):
    min_row: MinMax[int, Literal[False]]
    max_row: MinMax[int, Literal[False]]
    min_col: MinMax[int, Literal[False]]
    max_col: MinMax[int, Literal[False]]
    range_string: String[Literal[True]]
    worksheet: _WorkbookChild | ReadOnlyWorksheet | DummyWorksheet
    @overload
    def __init__(
        self,
        *,
        worksheet: _WorkbookChild | ReadOnlyWorksheet | DummyWorksheet | None = None,
        min_col: Unused = None,
        min_row: Unused = None,
        max_col: Unused = None,
        max_row: Unused = None,
        range_string: str,
    ) -> None: ...
    @overload
    def __init__(
        self,
        worksheet: _WorkbookChild | ReadOnlyWorksheet,
        min_col: ConvertibleToInt,
        min_row: ConvertibleToInt,
        max_col: ConvertibleToInt | None = None,
        max_row: ConvertibleToInt | None = None,
        range_string: str | None = None,
    ) -> None: ...
    def __len__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    @property
    def rows(self) -> Generator[Reference, None, None]: ...
    @property
    def cols(self) -> Generator[Reference, None, None]: ...
    def pop(self): ...
    @property
    def sheetname(self) -> str: ...



from braintree.attribute_getter import AttributeGetter
from braintree.receiver import Receiver
from braintree.sender import Sender

class Transfer(AttributeGetter):
    sender: Sender
    receiver: Receiver
    def __init__(self, attributes) -> None: ...



import datetime
from _typeshed import Incomplete
from collections.abc import Iterable, Mapping
from typing import Literal, NamedTuple, TypedDict, overload, type_check_only
from typing_extensions import NotRequired

from docker._types import ContainerWeightDevice, WaitContainerResponse
from docker.types import EndpointConfig
from docker.types.containers import DeviceRequest, LogConfig, Ulimit
from docker.types.daemon import CancellableStream
from docker.types.services import Mount

from .images import Image
from .resource import Collection, Model

@type_check_only
class _RestartPolicy(TypedDict):
    MaximumRetryCount: NotRequired[int]
    Name: NotRequired[Literal["always", "on-failure"]]

class Container(Model):
    @property
    def name(self) -> str | None: ...
    @property
    def image(self) -> Image | None: ...
    @property
    def labels(self): ...
    @property
    def status(self) -> str: ...
    @property
    def health(self) -> str: ...
    @property
    def ports(self) -> dict[Incomplete, Incomplete]: ...
    def attach(self, **kwargs): ...
    def attach_socket(self, **kwargs): ...
    def commit(self, repository: str | None = None, tag: str | None = None, **kwargs): ...
    def diff(self): ...
    def exec_run(
        self,
        cmd: str | list[str],
        stdout: bool = True,
        stderr: bool = True,
        stdin: bool = False,
        tty: bool = False,
        privileged: bool = False,
        user: str = "",
        detach: bool = False,
        stream: bool = False,
        socket: bool = False,
        environment=None,
        workdir=None,
        demux: bool = False,
    ) -> ExecResult: ...
    def export(self, chunk_size: int | None = 2097152) -> str: ...
    def get_archive(
        self, path: str, chunk_size: int | None = 2097152, encode_stream: bool = False
    ) -> tuple[Incomplete, Incomplete]: ...
    def kill(self, signal=None): ...
    @overload
    def logs(
        self,
        *,
        stdout: bool = True,
        stderr: bool = True,
        stream: Literal[True],
        timestamps: bool = False,
        tail: Literal["all"] | int = "all",
        since: datetime.datetime | float | None = None,
        follow: bool | None = None,
        until: datetime.datetime | float | None = None,
    ) -> CancellableStream[bytes]: ...
    @overload
    def logs(
        self,
        *,
        stdout: bool = True,
        stderr: bool = True,
        stream: Literal[False] = False,
        timestamps: bool = False,
        tail: Literal["all"] | int = "all",
        since: datetime.datetime | float | None = None,
        follow: bool | None = None,
        until: datetime.datetime | float | None = None,
    ) -> bytes: ...
    def pause(self) -> None: ...
    def put_archive(self, path: str, data) -> bool: ...
    def remove(self, *, v: bool = False, link: bool = False, force: bool = False) -> None: ...
    def rename(self, name: str): ...
    def resize(self, height: int, width: int): ...
    def restart(self, *, timeout: float | None = 10): ...
    def start(self) -> None: ...
    def stats(self, **kwargs): ...
    def stop(self, *, timeout: float | None = None) -> None: ...
    def top(self, *, ps_args: str | None = None) -> str: ...
    def unpause(self): ...
    def update(
        self,
        *,
        blkio_weight: int | None = None,
        cpu_period: int | None = None,
        cpu_quota: int | None = None,
        cpu_shares: int | None = None,
        cpuset_cpus: str | None = None,
        cpuset_mems: str | None = None,
        mem_limit: float | str | None = None,
        mem_reservation: float | str | None = None,
        memswap_limit: int | str | None = None,
        kernel_memory: int | str | None = None,
        restart_policy: _RestartPolicy | None = None,
    ): ...
    def wait(
        self, *, timeout: float | None = None, condition: Literal["not-running", "next-exit", "removed"] | None = None
    ) -> WaitContainerResponse: ...

class ContainerCollection(Collection[Container]):
    model: type[Container]
    @overload
    def run(
        self,
        image: str | Image,
        command: str | list[str] | None = None,
        stdout: bool = True,
        stderr: bool = False,
        remove: bool = False,
        *,
        auto_remove: bool = False,
        blkio_weight_device: list[ContainerWeightDevice] | None = None,
        blkio_weight: int | None = None,
        cap_add: list[str] | None = None,
        cap_drop: list[str] | None = None,
        cgroup_parent: str | None = None,
        cgroupns: Literal["private", "host"] | None = None,
        cpu_count: int | None = None,
        cpu_percent: int | None = None,
        cpu_period: int | None = None,
        cpu_quota: int | None = None,
        cpu_rt_period: int | None = None,
        cpu_rt_runtime: int | None = None,
        cpu_shares: int | None = None,
        cpuset_cpus: str | None = None,
        cpuset_mems: str | None = None,
        detach: Literal[False] = False,
        device_cgroup_rules: list[Incomplete] | None = None,
        device_read_bps: list[Mapping[str, str | int]] | None = None,
        device_read_iops: list[Mapping[str, str | int]] | None = None,
        device_write_bps: list[Mapping[str, str | int]] | None = None,
        device_write_iops: list[Mapping[str, str | int]] | None = None,
        devices: list[str] | None = None,
        device_requests: list[DeviceRequest] | None = None,
        dns: list[Incomplete] | None = None,
        dns_opt: list[Incomplete] | None = None,
        dns_search: list[Incomplete] | None = None,
        domainname: str | list[Incomplete] | None = None,
        entrypoint: str | list[str] | None = None,
        environment: dict[str, str] | list[str] | None = None,
        extra_hosts: dict[str, str] | None = None,
        group_add: Iterable[str | int] | None = None,
        healthcheck: dict[Incomplete, Incomplete] | None = None,
        hostname: str | None = None,
        init: bool | None = None,
        init_path: str | None = None,
        ipc_mode: str | None = None,
        isolation: str | None = None,
        kernel_memory: str | int | None = None,
        labels: dict[str, str] | list[str] | None = None,
        links: dict[str, str] | dict[str, None] | dict[str, str | None] | Iterable[tuple[str, str | None]] | None = None,
        log_config: LogConfig | None = None,
        lxc_conf: dict[Incomplete, Incomplete] | None = None,
        mac_address: str | None = None,
        mem_limit: str | int | None = None,
        mem_reservation: str | int | None = None,
        mem_swappiness: int | None = None,
        memswap_limit: str | int | None = None,
        mounts: list[Mount] | None = None,
        name: str | None = None,
        nano_cpus: int | None = None,
        network: str | None = None,
        network_disabled: bool = False,
        network_mode: str | None = None,
        networking_config: dict[str, EndpointConfig] | None = None,
        oom_kill_disable: bool = False,
        oom_score_adj: int | None = None,
        pid_mode: str | None = None,
        pids_limit: int | None = None,
        platform: str | None = None,
        ports: Mapping[str, int | list[int] | tuple[str, int] | None] | None = None,
        privileged: bool = False,
        publish_all_ports: bool = False,
        read_only: bool | None = None,
        restart_policy: _RestartPolicy | None = None,
        runtime: str | None = None,
        security_opt: list[str] | None = None,
        shm_size: str | int | None = None,
        stdin_open: bool = False,
        stop_signal: str | None = None,
        storage_opt: dict[Incomplete, Incomplete] | None = None,
        stream: bool = False,
        sysctls: dict[Incomplete, Incomplete] | None = None,
        tmpfs: dict[str, str] | None = None,
        tty: bool = False,
        ulimits: list[Ulimit] | None = None,
        use_config_proxy: bool | None = None,
        user: str | int | None = None,
        userns_mode: str | None = None,
        uts_mode: str | None = None,
        version: str | None = None,
        volume_driver: str | None = None,
        volumes: dict[str, dict[str, str]] | list[str] | None = None,
        volumes_from: list[str] | None = None,
        working_dir: str | None = None,
    ) -> bytes: ...  # TODO: This should return a stream, if `stream` is True
    @overload
    def run(
        self,
        image: str | Image,
        command: str | list[str] | None = None,
        stdout: bool = True,
        stderr: bool = False,
        remove: bool = False,
        *,
        auto_remove: bool = False,
        blkio_weight_device: list[ContainerWeightDevice] | None = None,
        blkio_weight: int | None = None,
        cap_add: list[str] | None = None,
        cap_drop: list[str] | None = None,
        cgroup_parent: str | None = None,
        cgroupns: Literal["private", "host"] | None = None,
        cpu_count: int | None = None,
        cpu_percent: int | None = None,
        cpu_period: int | None = None,
        cpu_quota: int | None = None,
        cpu_rt_period: int | None = None,
        cpu_rt_runtime: int | None = None,
        cpu_shares: int | None = None,
        cpuset_cpus: str | None = None,
        cpuset_mems: str | None = None,
        detach: Literal[True],
        device_cgroup_rules: list[Incomplete] | None = None,
        device_read_bps: list[Mapping[str, str | int]] | None = None,
        device_read_iops: list[Mapping[str, str | int]] | None = None,
        device_write_bps: list[Mapping[str, str | int]] | None = None,
        device_write_iops: list[Mapping[str, str | int]] | None = None,
        devices: list[str] | None = None,
        device_requests: list[DeviceRequest] | None = None,
        dns: list[Incomplete] | None = None,
        dns_opt: list[Incomplete] | None = None,
        dns_search: list[Incomplete] | None = None,
        domainname: str | list[Incomplete] | None = None,
        entrypoint: str | list[str] | None = None,
        environment: dict[str, str] | list[str] | None = None,
        extra_hosts: dict[str, str] | None = None,
        group_add: Iterable[str | int] | None = None,
        healthcheck: dict[Incomplete, Incomplete] | None = None,
        hostname: str | None = None,
        init: bool | None = None,
        init_path: str | None = None,
        ipc_mode: str | None = None,
        isolation: str | None = None,
        kernel_memory: str | int | None = None,
        labels: dict[str, str] | list[str] | None = None,
        links: dict[str, str] | dict[str, None] | dict[str, str | None] | Iterable[tuple[str, str | None]] | None = None,
        log_config: LogConfig | None = None,
        lxc_conf: dict[Incomplete, Incomplete] | None = None,
        mac_address: str | None = None,
        mem_limit: str | int | None = None,
        mem_reservation: str | int | None = None,
        mem_swappiness: int | None = None,
        memswap_limit: str | int | None = None,
        mounts: list[Mount] | None = None,
        name: str | None = None,
        nano_cpus: int | None = None,
        network: str | None = None,
        network_disabled: bool = False,
        network_mode: str | None = None,
        networking_config: dict[str, EndpointConfig] | None = None,
        oom_kill_disable: bool = False,
        oom_score_adj: int | None = None,
        pid_mode: str | None = None,
        pids_limit: int | None = None,
        platform: str | None = None,
        ports: Mapping[str, int | list[int] | tuple[str, int] | None] | None = None,
        privileged: bool = False,
        publish_all_ports: bool = False,
        read_only: bool | None = None,
        restart_policy: _RestartPolicy | None = None,
        runtime: str | None = None,
        security_opt: list[str] | None = None,
        shm_size: str | int | None = None,
        stdin_open: bool = False,
        stop_signal: str | None = None,
        storage_opt: dict[Incomplete, Incomplete] | None = None,
        stream: bool = False,
        sysctls: dict[Incomplete, Incomplete] | None = None,
        tmpfs: dict[str, str] | None = None,
        tty: bool = False,
        ulimits: list[Ulimit] | None = None,
        use_config_proxy: bool | None = None,
        user: str | int | None = None,
        userns_mode: str | None = None,
        uts_mode: str | None = None,
        version: str | None = None,
        volume_driver: str | None = None,
        volumes: dict[str, dict[str, str]] | list[str] | None = None,
        volumes_from: list[str] | None = None,
        working_dir: str | None = None,
    ) -> Container: ...
    def create(  # type:ignore[override]
        self,
        image: str | Image,
        command: str | list[str] | None = None,
        *,
        auto_remove: bool = False,
        blkio_weight_device: list[ContainerWeightDevice] | None = None,
        blkio_weight: int | None = None,
        cap_add: list[str] | None = None,
        cap_drop: list[str] | None = None,
        cgroup_parent: str | None = None,
        cgroupns: Literal["private", "host"] | None = None,
        cpu_count: int | None = None,
        cpu_percent: int | None = None,
        cpu_period: int | None = None,
        cpu_quota: int | None = None,
        cpu_rt_period: int | None = None,
        cpu_rt_runtime: int | None = None,
        cpu_shares: int | None = None,
        cpuset_cpus: str | None = None,
        cpuset_mems: str | None = None,
        detach: bool = False,
        device_cgroup_rules: list[Incomplete] | None = None,
        device_read_bps: list[Mapping[str, str | int]] | None = None,
        device_read_iops: list[Mapping[str, str | int]] | None = None,
        device_write_bps: list[Mapping[str, str | int]] | None = None,
        device_write_iops: list[Mapping[str, str | int]] | None = None,
        devices: list[str] | None = None,
        device_requests: list[DeviceRequest] | None = None,
        dns: list[Incomplete] | None = None,
        dns_opt: list[Incomplete] | None = None,
        dns_search: list[Incomplete] | None = None,
        domainname: str | list[Incomplete] | None = None,
        entrypoint: str | list[str] | None = None,
        environment: dict[str, str] | list[str] | None = None,
        extra_hosts: dict[str, str] | None = None,
        group_add: Iterable[str | int] | None = None,
        healthcheck: dict[Incomplete, Incomplete] | None = None,
        hostname: str | None = None,
        init: bool | None = None,
        init_path: str | None = None,
        ipc_mode: str | None = None,
        isolation: str | None = None,
        kernel_memory: str | int | None = None,
        labels: dict[str, str] | list[str] | None = None,
        links: dict[str, str] | dict[str, None] | dict[str, str | None] | Iterable[tuple[str, str | None]] | None = None,
        log_config: LogConfig | None = None,
        lxc_conf: dict[Incomplete, Incomplete] | None = None,
        mac_address: str | None = None,
        mem_limit: str | int | None = None,
        mem_reservation: str | int | None = None,
        mem_swappiness: int | None = None,
        memswap_limit: str | int | None = None,
        mounts: list[Mount] | None = None,
        name: str | None = None,
        nano_cpus: int | None = None,
        network: str | None = None,
        network_disabled: bool = False,
        network_mode: str | None = None,
        networking_config: dict[str, EndpointConfig] | None = None,
        oom_kill_disable: bool = False,
        oom_score_adj: int | None = None,
        pid_mode: str | None = None,
        pids_limit: int | None = None,
        platform: str | None = None,
        ports: Mapping[str, int | list[int] | tuple[str, int] | None] | None = None,
        privileged: bool = False,
        publish_all_ports: bool = False,
        read_only: bool | None = None,
        restart_policy: _RestartPolicy | None = None,
        runtime: str | None = None,
        security_opt: list[str] | None = None,
        shm_size: str | int | None = None,
        stdin_open: bool = False,
        stop_signal: str | None = None,
        storage_opt: dict[Incomplete, Incomplete] | None = None,
        stream: bool = False,
        sysctls: dict[Incomplete, Incomplete] | None = None,
        tmpfs: dict[str, str] | None = None,
        tty: bool = False,
        ulimits: list[Ulimit] | None = None,
        use_config_proxy: bool | None = None,
        user: str | int | None = None,
        userns_mode: str | None = None,
        uts_mode: str | None = None,
        version: str | None = None,
        volume_driver: str | None = None,
        volumes: dict[str, dict[str, str]] | list[str] | None = None,
        volumes_from: list[str] | None = None,
        working_dir: str | None = None,
    ) -> Container: ...
    def get(self, container_id: str) -> Container: ...
    def list(
        self,
        all: bool = False,
        before: str | None = None,
        filters=None,
        limit: int = -1,
        since: str | None = None,
        sparse: bool = False,
        ignore_removed: bool = False,
    ): ...
    def prune(self, filters=None): ...

RUN_CREATE_KWARGS: list[str]
RUN_HOST_CONFIG_KWARGS: list[str]

class ExecResult(NamedTuple):
    exit_code: Incomplete
    output: Incomplete



from collections.abc import Callable
from typing import Any, overload
from typing_extensions import Self, TypeAlias

from tensorflow import Tensor

class Regularizer:
    def get_config(self) -> dict[str, Any]: ...
    @classmethod
    def from_config(cls, config: dict[str, Any]) -> Self: ...
    def __call__(self, x: Tensor, /) -> Tensor: ...

_Regularizer: TypeAlias = str | dict[str, Any] | Regularizer | None  # noqa: Y047

@overload
def get(identifier: None) -> None: ...
@overload
def get(identifier: str | dict[str, Any] | Regularizer) -> Regularizer: ...
@overload
def get(identifier: Callable[[Tensor], Tensor]) -> Callable[[Tensor], Tensor]: ...
def __getattr__(name: str): ...  # incomplete module



from asgiref.typing import ASGIReceiveCallable, ASGISendCallable

from .consumer import _ChannelScope
from .utils import _ChannelApplication

class BaseMiddleware:
    inner: _ChannelApplication

    def __init__(self, inner: _ChannelApplication) -> None: ...
    async def __call__(
        self, scope: _ChannelScope, receive: ASGIReceiveCallable, send: ASGISendCallable
    ) -> _ChannelApplication: ...



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.charts.areas import PlotArea
from reportlab.graphics.widgetbase import PropHolder
from reportlab.lib.attrmap import *

__version__: Final[str]

class LineChartProperties(PropHolder): ...

class AbstractLineChart(PlotArea):
    def makeSwatchSample(self, rowNo, x, y, width, height): ...
    def getSeriesName(self, i, default=None): ...

class LineChart(AbstractLineChart): ...

class HorizontalLineChart(LineChart):
    strokeColor: Incomplete
    fillColor: Incomplete
    categoryAxis: Incomplete
    valueAxis: Incomplete
    data: Incomplete
    categoryNames: Incomplete
    lines: Incomplete
    useAbsolute: int
    groupSpacing: int
    lineLabels: Incomplete
    lineLabelFormat: Incomplete
    lineLabelArray: Incomplete
    lineLabelNudge: int
    joinedLines: int
    inFill: int
    reversePlotOrder: int
    def __init__(self) -> None: ...
    def demo(self): ...
    def calcPositions(self) -> None: ...
    def drawLabel(self, G, rowNo, colNo, x, y) -> None: ...
    def makeLines(self): ...
    def draw(self): ...

class _FakeGroup:
    def __init__(self) -> None: ...
    def add(self, what) -> None: ...
    def value(self): ...
    def sort(self) -> None: ...

class HorizontalLineChart3D(HorizontalLineChart):
    theta_x: float
    theta_y: float
    zDepth: int
    zSpace: int
    def calcPositions(self) -> None: ...
    def makeLines(self): ...

class VerticalLineChart(LineChart): ...

def sample1(): ...

class SampleHorizontalLineChart(HorizontalLineChart):
    def demo(self): ...
    def makeBackground(self): ...

def sample1a(): ...
def sample2(): ...
def sample3(): ...
def sampleCandleStick(): ...



from _typeshed import ReadableBuffer
from collections.abc import Sequence
from typing import Final
from xml.etree.ElementTree import (
    Element,
    ElementTree,
    ParseError as ParseError,
    XMLParser as _XMLParser,
    _FileRead,
    _IterParseIterator,
    _Target,
    tostring as tostring,
)

__origin__: Final = "xml.etree.ElementTree"

class DefusedXMLParser(_XMLParser):
    forbid_dtd: bool
    forbid_entities: bool
    forbid_external: bool
    def __init__(
        self,
        html: object = ...,  # argument is deprecated, if bool(html) is True you will get TypeError
        target: _Target | None = None,
        encoding: str | None = None,
        forbid_dtd: bool = False,
        forbid_entities: bool = True,
        forbid_external: bool = True,
    ) -> None: ...
    def defused_start_doctype_decl(self, name: str, sysid: str | None, pubid: str | None, has_internal_subset: bool) -> None: ...
    def defused_entity_decl(
        self,
        name: str,
        is_parameter_entity: bool,
        value: str | None,
        base: str | None,
        sysid: str,
        pubid: str | None,
        notation_name: str | None,
    ) -> None: ...
    def defused_unparsed_entity_decl(
        self, name: str, base: str | None, sysid: str, pubid: str | None, notation_name: str
    ) -> None: ...
    def defused_external_entity_ref_handler(
        self, context: str, base: str | None, sysid: str | None, pubid: str | None
    ) -> None: ...

XMLTreeBuilder = DefusedXMLParser
XMLParse = DefusedXMLParser
XMLParser = DefusedXMLParser

# wrapper to xml.etree.ElementTree.parse
def parse(
    source: _FileRead,
    parser: XMLParser | None = None,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> ElementTree: ...

# wrapper to xml.etree.ElementTree.iterparse
def iterparse(
    source: _FileRead,
    events: Sequence[str] | None = None,
    parser: XMLParser | None = None,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> _IterParseIterator: ...
def fromstring(
    text: str | ReadableBuffer, forbid_dtd: bool = False, forbid_entities: bool = True, forbid_external: bool = True
) -> Element: ...

XML = fromstring

__all__ = ["ParseError", "XML", "XMLParse", "XMLParser", "XMLTreeBuilder", "fromstring", "iterparse", "parse", "tostring"]



from _typeshed import Incomplete
from abc import ABCMeta
from typing_extensions import TypeAlias

from tensorflow import Tensor, TensorShape
from tensorflow._aliases import TensorCompatible
from tensorflow.dtypes import DType

_SparseTensorCompatible: TypeAlias = TensorCompatible | SparseTensor

class SparseTensor(metaclass=ABCMeta):
    @property
    def indices(self) -> Tensor: ...
    @property
    def values(self) -> Tensor: ...
    @property
    def dense_shape(self) -> Tensor: ...
    @property
    def shape(self) -> TensorShape: ...
    @property
    def dtype(self) -> DType: ...
    name: str
    def __init__(self, indices: TensorCompatible, values: TensorCompatible, dense_shape: TensorCompatible) -> None: ...
    def get_shape(self) -> TensorShape: ...
    # Many arithmetic operations are not directly supported. Some have alternatives like tf.sparse.add instead of +.
    def __div__(self, y: _SparseTensorCompatible) -> SparseTensor: ...
    def __truediv__(self, y: _SparseTensorCompatible) -> SparseTensor: ...
    def __mul__(self, y: _SparseTensorCompatible) -> SparseTensor: ...
    def __getattr__(self, name: str) -> Incomplete: ...

def __getattr__(name: str): ...  # incomplete module



from collections.abc import Callable, Mapping
from typing import Any
from typing_extensions import TypeAlias

from m3u8.httpclient import _HTTPClientProtocol
from m3u8.model import (
    M3U8,
    ContentSteering,
    DateRange,
    DateRangeList,
    IFramePlaylist,
    ImagePlaylist,
    Key,
    Media,
    MediaList,
    PartialSegment,
    PartialSegmentList,
    PartInformation,
    Playlist,
    PlaylistList,
    PreloadHint,
    RenditionReport,
    RenditionReportList,
    Segment,
    SegmentList,
    ServerControl,
    Skip,
    Start,
    Tiles,
)
from m3u8.parser import ParseError, parse

__all__ = (
    "M3U8",
    "Segment",
    "SegmentList",
    "PartialSegment",
    "PartialSegmentList",
    "Key",
    "Playlist",
    "IFramePlaylist",
    "Media",
    "MediaList",
    "PlaylistList",
    "Start",
    "RenditionReport",
    "RenditionReportList",
    "ServerControl",
    "Skip",
    "PartInformation",
    "PreloadHint",
    "DateRange",
    "DateRangeList",
    "ContentSteering",
    "ImagePlaylist",
    "Tiles",
    "loads",
    "load",
    "parse",
    "ParseError",
)

_CustomTagsParser: TypeAlias = Callable[[str, int, dict[str, Any], dict[str, Any]], object]

def loads(content: str, uri: str | None = None, custom_tags_parser: _CustomTagsParser | None = None) -> M3U8: ...
def load(
    uri: str,
    timeout: float | None = None,
    headers: Mapping[str, Any] = {},
    custom_tags_parser: _CustomTagsParser | None = None,
    http_client: _HTTPClientProtocol = ...,
    verify_ssl: bool = True,
) -> M3U8: ...



from _typeshed import SupportsItems
from collections.abc import Callable
from typing import Any, Generic, Literal, overload
from typing_extensions import Self, TypeVar

_T = TypeVar("_T")
_DefaultT = TypeVar("_DefaultT", default=None)
_NoneLiteral = TypeVar("_NoneLiteral", default=None)
_ScopeT = TypeVar("_ScopeT", Literal["request"], Literal["response"], None, default=None)
_ScopeT2 = TypeVar("_ScopeT2", Literal["request"], Literal["response"], None)

class UpdateDict(dict[str, Any]):
    updated: Callable[..., Any] | None
    updated_args: tuple[Any, ...] | None

class exists_property(Generic[_ScopeT]):
    @overload
    def __init__(self: exists_property[None], prop: str) -> None: ...
    @overload
    def __init__(self, prop: str, type: _ScopeT) -> None: ...
    @overload
    def __get__(self, obj: None, type: type[CacheControl[Any]] | None = None) -> Self: ...
    @overload
    def __get__(self: exists_property[None], obj: CacheControl[Any], type: type[CacheControl[Any]] | None = None) -> bool: ...
    @overload
    def __get__(self, obj: CacheControl[_ScopeT], type: type[CacheControl[Any]] | None = None) -> bool: ...
    @overload
    def __set__(self: exists_property[None], obj: CacheControl[Any], value: bool | None) -> None: ...
    @overload
    def __set__(self, obj: CacheControl[_ScopeT], value: bool | None) -> None: ...
    @overload
    def __delete__(self, obj: CacheControl[Any]) -> None: ...
    @overload
    def __delete__(self, obj: CacheControl[_ScopeT]) -> None: ...

class value_property(Generic[_T, _DefaultT, _NoneLiteral, _ScopeT]):
    def __init__(self, prop: str, default: _DefaultT = None, none: _NoneLiteral = None, type: _ScopeT = None) -> None: ...  # type: ignore[assignment]
    @overload
    def __get__(self, obj: None, type: type[CacheControl[Any]] | None = None) -> Self: ...
    @overload
    def __get__(
        self: value_property[_T, _DefaultT, _NoneLiteral, None],
        obj: CacheControl[Any] | None,
        type: type[CacheControl[Any]] | None = None,
    ) -> _T | _DefaultT | _NoneLiteral: ...
    @overload
    def __get__(
        self, obj: CacheControl[_ScopeT] | None, type: type[CacheControl[Any]] | None = None
    ) -> _T | _DefaultT | _NoneLiteral: ...
    @overload
    def __set__(
        self: value_property[_T, _DefaultT, _NoneLiteral, None],
        obj: CacheControl[Any],
        value: _T | _DefaultT | Literal[True] | None,
    ) -> None: ...
    @overload
    def __set__(self, obj: CacheControl[_ScopeT], value: _T | _DefaultT | Literal[True] | None) -> None: ...
    @overload
    def __delete__(self, obj: CacheControl[Any]) -> None: ...
    @overload
    def __delete__(self, obj: CacheControl[_ScopeT]) -> None: ...

class CacheControl(Generic[_ScopeT]):
    header_value: str
    update_dict: type[UpdateDict]
    properties: dict[str, Any]
    type: _ScopeT
    def __init__(self, properties: dict[str, Any], type: _ScopeT) -> None: ...
    @overload
    @classmethod
    def parse(
        cls, header: str, updates_to: Callable[[dict[str, Any]], Any] | None = None, type: None = None
    ) -> CacheControl[None]: ...
    @overload
    @classmethod
    def parse(cls, header: str, updates_to: Callable[[dict[str, Any]], Any] | None, type: _ScopeT2) -> CacheControl[_ScopeT2]: ...
    @overload
    @classmethod
    def parse(
        cls, header: str, updates_to: Callable[[dict[str, Any]], Any] | None = None, *, type: _ScopeT2
    ) -> CacheControl[_ScopeT2]: ...
    max_stale: value_property[int, None, Literal["*"], Literal["request"]]
    min_fresh: value_property[int, None, None, Literal["request"]]
    only_if_cached: exists_property[Literal["request"]]
    public: exists_property[Literal["response"]]
    private: value_property[str, None, Literal["*"], Literal["response"]]
    no_cache: value_property[str, None, Literal["*"], None]
    no_store: exists_property[None]
    no_transform: exists_property[None]
    must_revalidate: exists_property[Literal["response"]]
    proxy_revalidate: exists_property[Literal["response"]]
    max_age: value_property[int, None, Literal[-1], None]
    s_maxage: value_property[int, None, None, Literal["response"]]
    s_max_age = s_maxage
    stale_while_revalidate: value_property[int, None, None, Literal["response"]]
    stale_if_error: value_property[int, None, None, Literal["response"]]
    def copy(self) -> Self: ...

def serialize_cache_control(properties: SupportsItems[str, Any] | CacheControl[Any]) -> str: ...



from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Quota(SystemBackendMixin):
    def read_quota(self, name): ...
    def list_quotas(self): ...
    def create_or_update_quota(
        self, name, rate, path=None, interval=None, block_interval=None, role=None, rate_limit_type=None, inheritable=None
    ): ...
    def delete_quota(self, name): ...



from _typeshed import Incomplete
from logging import Logger
from typing import Final

log: Logger

USER_AGENT_KEY: Final = "HTTP_USER_AGENT"
X_FORWARDED_KEY: Final = "HTTP_X_FORWARDED_FOR"
REMOTE_ADDR_KEY: Final = "REMOTE_ADDR"
HOST_KEY: Final = "HTTP_HOST"
CONTENT_LENGTH_KEY: Final = "content-length"

class XRayMiddleware:
    get_response: Incomplete
    in_lambda_ctx: bool

    def __init__(self, get_response) -> None: ...
    def __call__(self, request): ...
    def process_exception(self, request, exception: Exception) -> None: ...



from collections.abc import Collection
from typing import Literal, overload
from typing_extensions import TypeAlias

from ._typing import GeoT
from .geometry import Point
from .lib import Geometry

__all__ = ["affine_transform", "rotate", "scale", "skew", "translate"]

_Origin: TypeAlias = Literal["center", "centroid"] | Point | tuple[float, float] | tuple[float, float, float]

def affine_transform(geom: GeoT, matrix: Collection[float]) -> GeoT: ...
@overload
def interpret_origin(geom: Geometry, origin: _Origin, ndim: Literal[2]) -> tuple[float, float]: ...
@overload
def interpret_origin(geom: Geometry, origin: _Origin, ndim: Literal[3]) -> tuple[float, float, float]: ...
@overload
def interpret_origin(geom: Geometry, origin: _Origin, ndim: int) -> tuple[float, float] | tuple[float, float, float]: ...
def rotate(geom: GeoT, angle: float, origin: _Origin = "center", use_radians: bool = False) -> GeoT: ...
def scale(geom: GeoT, xfact: float = 1.0, yfact: float = 1.0, zfact: float = 1.0, origin: _Origin = "center") -> GeoT: ...
def skew(geom: GeoT, xs: float = 0.0, ys: float = 0.0, origin: _Origin = "center", use_radians: bool = False) -> GeoT: ...
def translate(geom: GeoT, xoff: float = 0.0, yoff: float = 0.0, zoff: float = 0.0) -> GeoT: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["network_simplex"]

class _DataEssentialsAndFunctions:
    node_list: Incomplete
    node_indices: Incomplete
    node_demands: Incomplete
    edge_sources: Incomplete
    edge_targets: Incomplete
    edge_keys: Incomplete
    edge_indices: Incomplete
    edge_capacities: Incomplete
    edge_weights: Incomplete
    edge_count: Incomplete
    edge_flow: Incomplete
    node_potentials: Incomplete
    parent: Incomplete
    parent_edge: Incomplete
    subtree_size: Incomplete
    next_node_dft: Incomplete
    prev_node_dft: Incomplete
    last_descendent_dft: Incomplete

    def __init__(
        self, G: Graph[_Node], multigraph, demand: str = "demand", capacity: str = "capacity", weight: str = "weight"
    ) -> None: ...
    def initialize_spanning_tree(self, n, faux_inf) -> None: ...
    def find_apex(self, p, q): ...
    def trace_path(self, p, w): ...
    def find_cycle(self, i, p, q): ...
    def augment_flow(self, Wn, We, f) -> None: ...
    def trace_subtree(self, p) -> Generator[Incomplete, None, None]: ...
    def remove_edge(self, s, t) -> None: ...
    def make_root(self, q) -> None: ...
    def add_edge(self, i, p, q) -> None: ...
    def update_potentials(self, i, p, q) -> None: ...
    def reduced_cost(self, i): ...
    def find_entering_edges(self) -> Generator[Incomplete, None, None]: ...
    def residual_capacity(self, i, p): ...
    def find_leaving_edge(self, Wn, We): ...

@_dispatchable
def network_simplex(
    G: Graph[_Node], demand: str = "demand", capacity: str = "capacity", weight: str = "weight"
) -> tuple[int | Incomplete, dict[Incomplete, dict[Incomplete, Incomplete]]]: ...



from collections.abc import Callable
from typing import Protocol, type_check_only

from authlib.oauth2.rfc6749 import ClientMixin

@type_check_only
class _TokenGenerator(Protocol):
    def __call__(self, *, client: ClientMixin, grant_type: str, user, scope: str) -> str: ...

class BearerTokenGenerator:
    DEFAULT_EXPIRES_IN: int
    GRANT_TYPES_EXPIRES_IN: dict[str, int]
    access_token_generator: _TokenGenerator
    refresh_token_generator: _TokenGenerator
    expires_generator: Callable[[ClientMixin, str], int]
    def __init__(
        self,
        access_token_generator: _TokenGenerator,
        refresh_token_generator: _TokenGenerator | None = None,
        expires_generator: Callable[[ClientMixin, str], int] | None = None,
    ) -> None: ...
    @staticmethod
    def get_allowed_scope(client: ClientMixin, scope: str) -> str: ...
    def generate(
        self,
        grant_type: str,
        client: ClientMixin,
        user=None,
        scope: str | None = None,
        expires_in: int | None = None,
        include_refresh_token: bool = True,
    ) -> dict[str, str | int]: ...
    def __call__(
        self,
        grant_type: str,
        client: ClientMixin,
        user=None,
        scope: str | None = None,
        expires_in: int | None = None,
        include_refresh_token: bool = True,
    ) -> dict[str, str | int]: ...



from _typeshed import Unused
from collections.abc import Container
from typing import Any
from typing_extensions import TypeAlias

from peewee import Database, ModelBase, Proxy

# Is actually flask.Flask
_Flask: TypeAlias = Any

class FlaskDB:
    # Omitting undocumented base_model_class on purpose, use FlaskDB.Model instead
    database: Database | Proxy
    def __init__(
        self,
        app: _Flask | None = None,
        database: Database | Proxy | None = None,
        # Is actually type[ModelClass] but stubtest likely confuses with Model property
        # https://github.com/python/typeshed/pull/11731#issuecomment-2067694259
        model_class=...,
        excluded_routes: Container[str] | None = None,
    ) -> None: ...
    def init_app(self, app: _Flask) -> None: ...
    def get_model_class(self) -> type[ModelBase]: ...
    @property
    def Model(self) -> type[ModelBase]: ...
    def connect_db(self) -> None: ...
    def close_db(self, exc: Unused) -> None: ...



def compare_operation(dn, attribute, value, auto_encode, schema=None, validator=None, check_names: bool = False): ...
def compare_request_to_dict(request): ...
def compare_response_to_dict(response): ...



from __future__ import annotations

import array
from typing_extensions import assert_type

# Casting to bytes.
buf = b"abcdefg"
view = memoryview(buf).cast("c")
elm = view[0]
assert_type(elm, bytes)
assert_type(view[0:2], memoryview[bytes])

# Casting to a bool.
a = array.array("B", [0, 1, 2, 3])
mv = memoryview(a)
bool_mv = mv.cast("?")
assert_type(bool_mv[0], bool)
assert_type(bool_mv[0:2], memoryview[bool])


# Casting to a signed char.
a = array.array("B", [0, 1, 2, 3])
mv = memoryview(a)
signed_mv = mv.cast("b")
assert_type(signed_mv[0], int)
assert_type(signed_mv[0:2], memoryview[int])

# Casting to a signed short.
a = array.array("B", [0, 1, 2, 3])
mv = memoryview(a)
signed_mv = mv.cast("h")
assert_type(signed_mv[0], int)
assert_type(signed_mv[0:2], memoryview[int])

# Casting to a signed int.
a = array.array("B", [0, 1, 2, 3])
mv = memoryview(a)
signed_mv = mv.cast("i")
assert_type(signed_mv[0], int)
assert_type(signed_mv[0:2], memoryview[int])

# Casting to a signed long.
a = array.array("B", [0, 1, 2, 3])
mv = memoryview(a)
signed_mv = mv.cast("l")
assert_type(signed_mv[0], int)
assert_type(signed_mv[0:2], memoryview[int])

# Casting to a float.
a = array.array("B", [0, 1, 2, 3])
mv = memoryview(a)
float_mv = mv.cast("f")
assert_type(float_mv[0], float)
assert_type(float_mv[0:2], memoryview[float])

# An invalid literal should raise an error.
mv = memoryview(b"abc")
mv.cast("abc")  # type: ignore

mv.index(42)  # type: ignore
mv.count(42)  # type: ignore



from collections.abc import Collection, Mapping

import numpy as np
from networkx._typing import Array2D
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["pagerank", "google_matrix"]

@_dispatchable
def pagerank(
    G: Graph[_Node],
    alpha: float | None = 0.85,
    personalization: Mapping[_Node, float] | None = None,
    max_iter: int | None = 100,
    tol: float | None = 1e-06,
    nstart: Mapping[_Node, float] | None = None,
    weight: str | None = "weight",
    dangling: Mapping[_Node, float] | None = None,
) -> dict[_Node, float]: ...
@_dispatchable
def google_matrix(
    G: Graph[_Node],
    alpha: float = 0.85,
    personalization: Mapping[_Node, float] | None = None,
    nodelist: Collection[_Node] | None = None,
    weight: str | None = "weight",
    dangling: Mapping[_Node, float] | None = None,
) -> Array2D[np.float64]: ...



from authlib.jose import JWTClaims

class JWTAccessTokenClaims(JWTClaims):
    def validate(self, *, now=None, leeway: int = 0) -> None: ...  # type: ignore[override]
    def validate_typ(self) -> None: ...
    def validate_client_id(self): ...
    def validate_auth_time(self) -> None: ...
    def validate_acr(self): ...
    def validate_amr(self) -> None: ...
    def validate_scope(self): ...
    def validate_groups(self): ...
    def validate_roles(self): ...
    def validate_entitlements(self): ...



from _typeshed import Incomplete

from braintree.resource import Resource

class RevokedPaymentMethodMetadata(Resource):
    revoked_payment_method: Incomplete
    customer_id: Incomplete
    token: Incomplete
    def __init__(self, gateway, attributes) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing import NamedTuple

__all__ = ["not_implemented_for", "open_file", "nodes_or_number", "np_random_state", "py_random_state", "argmap"]

def not_implemented_for(*graph_types): ...
def open_file(path_arg, mode: str = "r"): ...
def nodes_or_number(which_args): ...
def np_random_state(random_state_argument): ...
def py_random_state(random_state_argument): ...

class argmap:
    def __init__(self, func, *args, try_finally: bool = False) -> None: ...
    def __call__(self, f) -> Callable[..., Incomplete]: ...
    def compile(self, f) -> Callable[..., Incomplete]: ...
    def assemble(self, f): ...
    @classmethod
    def signature(cls, f): ...

    class Signature(NamedTuple):
        name: Incomplete
        signature: Incomplete
        def_sig: Incomplete
        call_sig: Incomplete
        names: Incomplete
        n_positional: Incomplete
        args: Incomplete
        kwargs: Incomplete



from _typeshed import Incomplete

from .shapes import Path, UserNode

class SvgPath(Path, UserNode):
    fillColor: Incomplete
    def __init__(self, s, isClipPath: int = 0, autoclose=None, fillMode=0, **kw) -> None: ...
    def provideNode(self): ...

__all__ = ("SvgPath",)
